@@ -17,7 +17,7 @@
 
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: channel.c,v 1.56.2.3 2007/04/03 22:08:52 fw Exp $";
+static char UNUSED id[] = "$Id: channel.c,v 1.65 2008/02/05 16:31:35 fw Exp $";
 
 #include "imp.h"
 #include <assert.h>
@@ -56,7 +56,7 @@ static CL2CHAN *My_Cl2Chan;
 
 static CL2CHAN *Get_Cl2Chan PARAMS(( CHANNEL *Chan, CLIENT *Client ));
 static CL2CHAN *Add_Client PARAMS(( CHANNEL *Chan, CLIENT *Client ));
-static bool Remove_Client PARAMS(( int Type, CHANNEL *Chan, CLIENT *Client, CLIENT *Origin, char *Reason, bool InformServer ));
+static bool Remove_Client PARAMS(( int Type, CHANNEL *Chan, CLIENT *Client, CLIENT *Origin, const char *Reason, bool InformServer ));
 static CL2CHAN *Get_First_Cl2Chan PARAMS(( CLIENT *Client, CHANNEL *Chan ));
 static CL2CHAN *Get_Next_Cl2Chan PARAMS(( CL2CHAN *Start, CLIENT *Client, CHANNEL *Chan ));
 static bool Delete_Channel PARAMS(( CHANNEL *Chan ));
@@ -201,25 +201,38 @@ Channel_Join( CLIENT *Client, char *Name )
 } /* Channel_Join */
 
 
+/**
+ * Remove client from channel.
+ * This function lets a client lead a channel. First, the function checks
+ * if the channel exists and the client is a member of it and sends out
+ * appropriate error messages if not. The real work is done by the function
+ * Remove_Client().
+ */
 GLOBAL bool
-Channel_Part( CLIENT *Client, CLIENT *Origin, char *Name, char *Reason )
+Channel_Part(CLIENT * Client, CLIENT * Origin, const char *Name, const char *Reason)
 {
 	CHANNEL *chan;
 
-	assert( Client != NULL );
-	assert( Name != NULL );
-	assert( Reason != NULL );
+	assert(Client != NULL);
+	assert(Name != NULL);
+	assert(Reason != NULL);
 
-	chan = Channel_Search( Name );
-	if(( ! chan ) || ( ! Get_Cl2Chan( chan, Client )))
-	{
-		IRC_WriteStrClient( Client, ERR_NOSUCHCHANNEL_MSG, Client_ID( Client ), Name );
+	chan = Channel_Search(Name);
+	if (!chan) {
+		IRC_WriteStrClient(Client, ERR_NOSUCHCHANNEL_MSG,
+				   Client_ID(Client), Name);
+		return false;
+	}
+	if (!Get_Cl2Chan(chan, Client)) {
+		IRC_WriteStrClient(Client, ERR_NOTONCHANNEL_MSG,
+				   Client_ID(Client), Name);
 		return false;
 	}
 
-	/* User aus Channel entfernen */
-	if( ! Remove_Client( REMOVE_PART, chan, Client, Origin, Reason, true)) return false;
-	else return true;
+	if (!Remove_Client(REMOVE_PART, chan, Client, Origin, Reason, true))
+		return false;
+	else
+		return true;
 } /* Channel_Part */
 
 
@@ -388,7 +401,7 @@ Channel_Next( CHANNEL *Chan )
 
 
 GLOBAL CHANNEL *
-Channel_Search( char *Name )
+Channel_Search( const char *Name )
 {
 	/* Channel-Struktur suchen */
 	
@@ -602,7 +615,7 @@ Channel_IsMemberOf( CHANNEL *Chan, CLIENT *Client )
 
 	assert( Chan != NULL );
 	assert( Client != NULL );
-	return Get_Cl2Chan(Chan, Client);
+	return Get_Cl2Chan(Chan, Client) != NULL;
 } /* Channel_IsMemberOf */
 
 
@@ -695,38 +708,66 @@ Channel_SetMaxUsers(CHANNEL *Chan, unsigned long Count)
 } /* Channel_SetMaxUsers */
 
 
-GLOBAL bool
-Channel_Write( CHANNEL *Chan, CLIENT *From, CLIENT *Client, char *Text )
+static bool
+Can_Send_To_Channel(CHANNEL *Chan, CLIENT *From)
 {
-	bool is_member, has_voice, is_op, ok;
+	bool is_member, has_voice, is_op;
 
-	/* Okay, target is a channel */
 	is_member = has_voice = is_op = false;
-	if( Channel_IsMemberOf( Chan, From ))
-	{
+
+	if (Channel_IsMemberOf(Chan, From)) {
 		is_member = true;
-		if( strchr( Channel_UserModes( Chan, From ), 'v' )) has_voice = true;
-		if( strchr( Channel_UserModes( Chan, From ), 'o' )) is_op = true;
+		if (strchr(Channel_UserModes(Chan, From), 'v'))
+			has_voice = true;
+		if (strchr(Channel_UserModes(Chan, From), 'o'))
+			is_op = true;
 	}
 
-	/* Is the client allowed to write to channel? */
-	ok = true;
-	if( strchr( Channel_Modes( Chan ), 'n' ) && ( ! is_member )) ok = false;
-	if( strchr( Channel_Modes( Chan ), 'm' ) && ( ! is_op ) && ( ! has_voice )) ok = false;
+	/*
+	 * Is the client allowed to write to channel?
+	 *
+	 * If channel mode n set: non-members cannot send to channel.
+	 * If channel mode m set: need voice.
+	 */
+	if (strchr(Channel_Modes(Chan), 'n') && !is_member)
+		return false;
 
-	/* Is the client banned? */
-	if( Lists_Check(&Chan->list_bans, From))
-	{
-		/* Client is banned, but is he channel operator or has voice? */
-		if(( ! has_voice ) && ( ! is_op )) ok = false;
-	}
+	if (is_op || has_voice)
+		return true;
 
-	if( ! ok ) return IRC_WriteStrClient( From, ERR_CANNOTSENDTOCHAN_MSG, Client_ID( From ), Channel_Name( Chan ));
+	if (strchr(Channel_Modes(Chan), 'm'))
+		return false;
 
-	/* Send text */
-	if( Client_Conn( From ) > NONE ) Conn_UpdateIdle( Client_Conn( From ));
-	return IRC_WriteStrChannelPrefix( Client, Chan, From, true, "PRIVMSG %s :%s", Channel_Name( Chan ), Text );
-} /* Channel_Write */
+	return !Lists_Check(&Chan->list_bans, From);
+}
+
+
+GLOBAL bool
+Channel_Write(CHANNEL *Chan, CLIENT *From, CLIENT *Client, const char *Text)
+{
+	if (!Can_Send_To_Channel(Chan, From))
+		return IRC_WriteStrClient(From, ERR_CANNOTSENDTOCHAN_MSG, Client_ID(From), Channel_Name(Chan));
+
+	if (Client_Conn(From) > NONE)
+		Conn_UpdateIdle(Client_Conn(From));
+
+	return IRC_WriteStrChannelPrefix(Client, Chan, From, true,
+			"PRIVMSG %s :%s", Channel_Name(Chan), Text);
+}
+
+
+GLOBAL bool
+Channel_Notice(CHANNEL *Chan, CLIENT *From, CLIENT *Client, const char *Text)
+{
+	if (!Can_Send_To_Channel(Chan, From))
+		return true; /* no error, see RFC 2812 */
+
+	if (Client_Conn(From) > NONE)
+		Conn_UpdateIdle(Client_Conn(From));
+
+	return IRC_WriteStrChannelPrefix(Client, Chan, From, true,
+			"NOTICE %s :%s", Channel_Name(Chan), Text);
+}
 
 
 GLOBAL CHANNEL *
@@ -801,7 +842,7 @@ Add_Client( CHANNEL *Chan, CLIENT *Client )
 
 
 static bool
-Remove_Client( int Type, CHANNEL *Chan, CLIENT *Client, CLIENT *Origin, char *Reason, bool InformServer )
+Remove_Client( int Type, CHANNEL *Chan, CLIENT *Client, CLIENT *Origin, const char *Reason, bool InformServer )
 {
 	CL2CHAN *cl2chan, *last_cl2chan;
 	CHANNEL *c;
@@ -875,7 +916,7 @@ Remove_Client( int Type, CHANNEL *Chan, CLIENT *Client, CLIENT *Origin, char *Re
 	{
 		if( ! Get_First_Cl2Chan( NULL, Chan )) Delete_Channel( Chan );
 	}
-		
+
 	return true;
 } /* Remove_Client */
 
