@@ -14,7 +14,7 @@
 
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: conf.c,v 1.92.2.4 2007/04/03 22:08:52 fw Exp $";
+static char UNUSED id[] = "$Id: conf.c,v 1.105 2008/03/18 20:12:47 fw Exp $";
 
 #include "imp.h"
 #include <assert.h>
@@ -56,9 +56,21 @@ static CONF_SERVER New_Server;
 static int New_Server_Idx;
 
 
+#ifdef WANT_IPV6
+/*
+ * these options appeared in ngircd 0.12; they are here
+ * for backwards compatibility. They should be removed
+ * in the future. Instead of setting these options,
+ * the "Listen" option should be set accordingly.
+ */
+static bool Conf_ListenIPv6;
+static bool Conf_ListenIPv4;
+#endif
+
+
 static void Set_Defaults PARAMS(( bool InitServers ));
-static void Read_Config PARAMS(( void ));
-static void Validate_Config PARAMS(( bool TestOnly, bool Rehash ));
+static bool Read_Config PARAMS(( bool ngircd_starting ));
+static bool Validate_Config PARAMS(( bool TestOnly, bool Rehash ));
 
 static void Handle_GLOBAL PARAMS(( int Line, char *Var, char *Arg ));
 static void Handle_OPERATOR PARAMS(( int Line, char *Var, char *Arg ));
@@ -134,24 +146,33 @@ ports_parse(array *a, int Line, char *Arg)
 GLOBAL void
 Conf_Init( void )
 {
-	Set_Defaults( true );
-	Read_Config( );
+	Read_Config( true );
 	Validate_Config(false, false);
 } /* Config_Init */
 
 
-GLOBAL void
+GLOBAL bool
 Conf_Rehash( void )
 {
-	Set_Defaults( false );
-	Read_Config( );
+	if (!Read_Config(false))
+		return false;
 	Validate_Config(false, true);
 
 	/* Update CLIENT structure of local server */
 	Client_SetInfo(Client_ThisServer(), Conf_ServerInfo);
+	return true;
 } /* Config_Rehash */
 
 
+static const char*
+yesno_to_str(int boolean_value)
+{
+	if (boolean_value)
+		return "yes";
+	return "no";
+}
+
+
 GLOBAL int
 Conf_Test( void )
 {
@@ -161,12 +182,14 @@ Conf_Test( void )
 	struct group *grp;
 	unsigned int i;
 	char *topic;
+	bool config_valid;
 
 	Use_Log = false;
-	Set_Defaults( true );
 
-	Read_Config( );
-	Validate_Config(true, false);
+	if (! Read_Config(true))
+		return 1;
+
+	config_valid = Validate_Config(true, false);
 
 	/* If stdin and stdout ("you can read our nice message and we can
 	 * read in your keypress") are valid tty's, wait for a key: */
@@ -191,8 +214,7 @@ Conf_Test( void )
 	fputs("  Ports = ", stdout);
 
 	ports_puts(&Conf_ListenPorts);
-
-	printf( "  Listen = %s\n", Conf_ListenAddress );
+	printf("  Listen = %s\n", Conf_ListenAddress);
 	pwd = getpwuid( Conf_UID );
 	if( pwd ) printf( "  ServerUID = %s\n", pwd->pw_name );
 	else printf( "  ServerUID = %ld\n", (long)Conf_UID );
@@ -202,12 +224,24 @@ Conf_Test( void )
 	printf( "  PingTimeout = %d\n", Conf_PingTimeout );
 	printf( "  PongTimeout = %d\n", Conf_PongTimeout );
 	printf( "  ConnectRetry = %d\n", Conf_ConnectRetry );
-	printf( "  OperCanUseMode = %s\n", Conf_OperCanMode == true ? "yes" : "no" );
-	printf( "  OperServerMode = %s\n", Conf_OperServerMode == true? "yes" : "no" );
-	printf( "  PredefChannelsOnly = %s\n", Conf_PredefChannelsOnly == true ? "yes" : "no" );
-	printf( "  MaxConnections = %ld\n", Conf_MaxConnections>0 ? Conf_MaxConnections : -1);
-	printf( "  MaxConnectionsIP = %d\n", Conf_MaxConnectionsIP>0 ? Conf_MaxConnectionsIP : -1);
-	printf( "  MaxJoins = %d\n\n", Conf_MaxJoins>0 ? Conf_MaxJoins : -1);
+	printf( "  OperCanUseMode = %s\n", yesno_to_str(Conf_OperCanMode));
+	printf( "  OperServerMode = %s\n", yesno_to_str(Conf_OperServerMode));
+	printf( "  PredefChannelsOnly = %s\n", yesno_to_str(Conf_PredefChannelsOnly));
+	printf( "  NoDNS = %s\n", yesno_to_str(Conf_NoDNS));
+
+#ifdef WANT_IPV6
+	/* both are deprecated, only mention them if their default value changed. */
+	if (!Conf_ListenIPv6)
+		puts("  ListenIPv6 = no");
+	if (!Conf_ListenIPv4)
+		puts("  ListenIPv4 = no");
+	printf("  ConnectIPv4 = %s\n", yesno_to_str(Conf_ConnectIPv6));
+	printf("  ConnectIPv6 = %s\n", yesno_to_str(Conf_ConnectIPv4));
+#endif
+	printf( "  MaxConnections = %ld\n", Conf_MaxConnections);
+	printf( "  MaxConnectionsIP = %d\n", Conf_MaxConnectionsIP);
+	printf( "  MaxJoins = %d\n", Conf_MaxJoins>0 ? Conf_MaxJoins : -1);
+	printf( "  MaxNickLength = %u\n\n", Conf_MaxNickLength - 1);
 
 	for( i = 0; i < Conf_Oper_Count; i++ ) {
 		if( ! Conf_Oper[i].name[0] ) continue;
@@ -230,7 +264,8 @@ Conf_Test( void )
 		printf( "  Port = %u\n", (unsigned int)Conf_Server[i].port );
 		printf( "  MyPassword = %s\n", Conf_Server[i].pwd_in );
 		printf( "  PeerPassword = %s\n", Conf_Server[i].pwd_out );
-		printf( "  Group = %d\n\n", Conf_Server[i].group );
+		printf( "  Group = %d\n", Conf_Server[i].group );
+		printf( "  Passive = %s\n\n", Conf_Server[i].flags & CONF_SFLAG_DISABLED ? "yes" : "no");
 	}
 
 	for( i = 0; i < Conf_Channel_Count; i++ ) {
@@ -247,7 +282,7 @@ Conf_Test( void )
 		printf( "  Topic = %s\n\n", topic ? topic : "");
 	}
 
-	return 0;
+	return (config_valid ? 0 : 1);
 } /* Conf_Test */
 
 
@@ -336,6 +371,24 @@ Conf_EnableServer( char *Name, UINT16 Port )
 } /* Conf_EnableServer */
 
 
+GLOBAL bool
+Conf_EnablePassiveServer(const char *Name)
+{
+	/* Enable specified server */
+	int i;
+
+	assert( Name != NULL );
+	for (i = 0; i < MAX_SERVERS; i++) {
+		if ((strcasecmp( Conf_Server[i].name, Name ) == 0) && (Conf_Server[i].port > 0)) {
+			/* BINGO! Enable server */
+			Conf_Server[i].flags &= ~CONF_SFLAG_DISABLED;
+			return true;
+		}
+	}
+	return false;
+} /* Conf_EnablePassiveServer */
+
+
 GLOBAL bool
 Conf_DisableServer( char *Name )
 {
@@ -412,8 +465,8 @@ Set_Defaults( bool InitServers )
 
 	strlcpy( Conf_PidFile, PID_FILE, sizeof( Conf_PidFile ));
 
-	strcpy( Conf_ListenAddress, "" );
-
+	free(Conf_ListenAddress);
+	Conf_ListenAddress = NULL;
 	Conf_UID = Conf_GID = 0;
 
 	Conf_PingTimeout = 120;
@@ -425,20 +478,28 @@ Set_Defaults( bool InitServers )
 	Conf_Channel_Count = 0;
 
 	Conf_OperCanMode = false;
+	Conf_NoDNS = false;
 	Conf_PredefChannelsOnly = false;
 	Conf_OperServerMode = false;
 
-	Conf_MaxConnections = -1;
+	Conf_ConnectIPv4 = true;
+	Conf_ConnectIPv6 = true;
+#ifdef WANT_IPV6
+	Conf_ListenIPv4 = true;
+	Conf_ListenIPv6 = true;
+#endif
+	Conf_MaxConnections = 0;
 	Conf_MaxConnectionsIP = 5;
 	Conf_MaxJoins = 10;
+	Conf_MaxNickLength = CLIENT_NICK_LEN_DEFAULT;
 
 	/* Initialize server configuration structures */
 	if( InitServers ) for( i = 0; i < MAX_SERVERS; Init_Server_Struct( &Conf_Server[i++] ));
 } /* Set_Defaults */
 
 
-static void
-Read_Config( void )
+static bool
+Read_Config( bool ngircd_starting )
 {
 	/* Read configuration file. */
 
@@ -453,10 +514,14 @@ Read_Config( void )
 		/* No configuration file found! */
 		Config_Error( LOG_ALERT, "Can't read configuration \"%s\": %s",
 					NGIRCd_ConfFile, strerror( errno ));
+		if (!ngircd_starting)
+			return false;
 		Config_Error( LOG_ALERT, "%s exiting due to fatal errors!", PACKAGE_NAME );
 		exit( 1 );
 	}
 
+	Set_Defaults( ngircd_starting );
+
 	Config_Error( LOG_INFO, "Reading configuration from \"%s\" ...", NGIRCd_ConfFile );
 
 	/* Clean up server configuration structure: mark all already
@@ -603,6 +668,24 @@ Read_Config( void )
 			exit( 1 );
 		}
 	}
+
+	if (!Conf_ListenAddress) {
+		/* no Listen addresses configured, use default */
+#ifdef WANT_IPV6
+		/* Conf_ListenIPv6/4 should no longer be used */
+		if (Conf_ListenIPv6 && Conf_ListenIPv4)
+			Conf_ListenAddress = strdup_warn("::,0.0.0.0");
+		else if (Conf_ListenIPv6)
+			Conf_ListenAddress = strdup_warn("::");
+		else
+#endif
+		Conf_ListenAddress = strdup_warn("0.0.0.0");
+	}
+	if (!Conf_ListenAddress) {
+		Config_Error(LOG_ALERT, "%s exiting due to fatal errors!", PACKAGE_NAME);
+		exit(1);
+	}
+	return true;
 } /* Read_Config */
 
 
@@ -617,6 +700,27 @@ Check_ArgIsTrue( const char *Arg )
 } /* Check_ArgIsTrue */
 
 
+static unsigned int Handle_MaxNickLength(int Line, const char *Arg)
+{
+	unsigned new;
+
+	new = (unsigned) atoi(Arg) + 1;
+	if (new > CLIENT_NICK_LEN) {
+		Config_Error(LOG_WARNING,
+			     "%s, line %d: Value of \"MaxNickLength\" exceeds %u!",
+			     NGIRCd_ConfFile, Line, CLIENT_NICK_LEN - 1);
+		return CLIENT_NICK_LEN;
+	}
+	if (new < 2) {
+		Config_Error(LOG_WARNING,
+			     "%s, line %d: Value of \"MaxNickLength\" must be at least 1!",
+			     NGIRCd_ConfFile, Line);
+		return 2;
+	}
+	return new;
+} /* Handle_MaxNickLength */
+
+
 static void
 Handle_GLOBAL( int Line, char *Var, char *Arg )
 {
@@ -764,6 +868,46 @@ Handle_GLOBAL( int Line, char *Var, char *Arg )
 		Conf_PredefChannelsOnly = Check_ArgIsTrue( Arg );
 		return;
 	}
+	if( strcasecmp( Var, "NoDNS" ) == 0 ) {
+		/* don't do reverse dns lookups when clients connect? */
+		Conf_NoDNS = Check_ArgIsTrue( Arg );
+		return;
+	}
+#ifdef WANT_IPV6
+	/* the default setting for all the WANT_IPV6 special options is 'true' */
+	if (strcasecmp(Var, "ListenIPv6") == 0) { /* DEPRECATED, option appeared in 0.12.0 */
+		/*
+		 * listen on ipv6 sockets, if available?
+		 * Deprecated use "Listen = 0.0.0.0" (or, rather, do not list "::")
+		 */
+		Conf_ListenIPv6 = Check_ArgIsTrue( Arg );
+		Config_Error(LOG_WARNING, "%s, line %d: %s=%s is deprecated, %sinclude '::' in \"Listen =\" option instead",
+				NGIRCd_ConfFile, Line, Var, yesno_to_str(Conf_ListenIPv6), Conf_ListenIPv6 ? " ":"do not ");
+		return;
+	}
+	if (strcasecmp(Var, "ListenIPv4") == 0) { /* DEPRECATED, option appeared in 0.12.0 */
+		/*
+		 * listen on ipv4 sockets, if available?
+		 * this allows "ipv6-only" setups
+		 * Deprecated use "Listen = ::" (or, rather, do not list "0.0.0.0")
+		 */
+		Conf_ListenIPv4 = Check_ArgIsTrue( Arg );
+		Config_Error(LOG_WARNING, "%s, line %d: %s=%s is deprecated, %sinclude '0.0.0.0' in \"Listen =\" option instead",
+				NGIRCd_ConfFile, Line, Var, yesno_to_str(Conf_ListenIPv4), Conf_ListenIPv4 ? " ":"do not ");
+		return;
+	}
+	if( strcasecmp( Var, "ConnectIPv6" ) == 0 ) {
+		/* connect to other hosts using ipv6, if they have an AAAA record? */
+		Conf_ConnectIPv6 = Check_ArgIsTrue( Arg );
+		return;
+	}
+	if( strcasecmp( Var, "ConnectIPv4" ) == 0 ) {
+		/* connect to other hosts using ipv4.
+		 * again, this can be used for ipv6-only setups */
+		Conf_ConnectIPv4 = Check_ArgIsTrue( Arg );
+		return;
+	}
+#endif
 	if( strcasecmp( Var, "OperCanUseMode" ) == 0 ) {
 		/* Are IRC operators allowed to use MODE in channels they aren't Op in? */
 		Conf_OperCanMode = Check_ArgIsTrue( Arg );
@@ -775,7 +919,7 @@ Handle_GLOBAL( int Line, char *Var, char *Arg )
 		return;
 	}
 	if( strcasecmp( Var, "MaxConnections" ) == 0 ) {
-		/* Maximum number of connections. Values <= 0 are equal to "no limit". */
+		/* Maximum number of connections. 0 -> "no limit". */
 #ifdef HAVE_ISDIGIT
 		if( ! isdigit( (int)*Arg )) Config_Error_NaN( Line, Var);
 		else
@@ -784,7 +928,7 @@ Handle_GLOBAL( int Line, char *Var, char *Arg )
 		return;
 	}
 	if( strcasecmp( Var, "MaxConnectionsIP" ) == 0 ) {
-		/* Maximum number of simultaneous connections from one IP. Values <= 0 -> "no limit" */
+		/* Maximum number of simultaneous connections from one IP. 0 -> "no limit" */
 #ifdef HAVE_ISDIGIT
 		if( ! isdigit( (int)*Arg )) Config_Error_NaN( Line, Var );
 		else
@@ -793,7 +937,7 @@ Handle_GLOBAL( int Line, char *Var, char *Arg )
 		return;
 	}
 	if( strcasecmp( Var, "MaxJoins" ) == 0 ) {
-		/* Maximum number of channels a user can join. Values <= 0 are equal to "no limit". */
+		/* Maximum number of channels a user can join. 0 -> "no limit". */
 #ifdef HAVE_ISDIGIT
 		if( ! isdigit( (int)*Arg )) Config_Error_NaN( Line, Var );
 		else
@@ -801,16 +945,33 @@ Handle_GLOBAL( int Line, char *Var, char *Arg )
 		Conf_MaxJoins = atoi( Arg );
 		return;
 	}
-	if( strcasecmp( Var, "Listen" ) == 0 ) {
-		/* IP-Address to bind sockets */
-		len = strlcpy( Conf_ListenAddress, Arg, sizeof( Conf_ListenAddress ));
-		if (len >= sizeof( Conf_ListenAddress ))
-			Config_Error_TooLong( Line, Var );
+	if( strcasecmp( Var, "MaxNickLength" ) == 0 ) {
+		/* Maximum length of a nick name; must be same on all servers
+		 * within the IRC network! */
+		Conf_MaxNickLength = Handle_MaxNickLength(Line, Arg);
 		return;
 	}
 
-	Config_Error( LOG_ERR, "%s, line %d (section \"Global\"): Unknown variable \"%s\"!",
-								NGIRCd_ConfFile, Line, Var );
+	if( strcasecmp( Var, "Listen" ) == 0 ) {
+		/* IP-Address to bind sockets */
+		if (Conf_ListenAddress) {
+			Config_Error(LOG_ERR, "Multiple Listen= options, ignoring: %s", Arg);
+			return;
+		}
+		Conf_ListenAddress = strdup_warn(Arg);
+		/*
+		 * if allocation fails, we're in trouble:
+		 * we cannot ignore the error -- otherwise ngircd
+		 * would listen on all interfaces.
+		 */
+		if (!Conf_ListenAddress) {
+			Config_Error(LOG_ALERT, "%s exiting due to fatal errors!", PACKAGE_NAME);
+			exit(1);
+		}
+		return;
+	}
+	Config_Error(LOG_ERR, "%s, line %d (section \"Global\"): Unknown variable \"%s\"!",
+								NGIRCd_ConfFile, Line, Var);
 } /* Handle_GLOBAL */
 
 
@@ -881,6 +1042,14 @@ Handle_SERVER( int Line, char *Var, char *Arg )
 			Config_Error_TooLong( Line, Var );
 		return;
 	}
+	if (strcasecmp(Var, "Bind") == 0) {
+		if (ng_ipaddr_init(&New_Server.bind_addr, Arg, 0))
+			return;
+
+		Config_Error(LOG_ERR, "%s, line %d (section \"Server\"): Can't parse IP address \"%s\"",
+				NGIRCd_ConfFile, Line, Arg);
+		return;
+	}
 	if( strcasecmp( Var, "MyPassword" ) == 0 ) {
 		/* Password of this server which is sent to the peer */
 		if (*Arg == ':') {
@@ -920,6 +1089,11 @@ Handle_SERVER( int Line, char *Var, char *Arg )
 		New_Server.group = atoi( Arg );
 		return;
 	}
+	if( strcasecmp( Var, "Passive" ) == 0 ) {
+		if (Check_ArgIsTrue(Arg))
+			New_Server.flags |= CONF_SFLAG_DISABLED;
+		return;
+	}
 	
 	Config_Error( LOG_ERR, "%s, line %d (section \"Server\"): Unknown variable \"%s\"!",
 								NGIRCd_ConfFile, Line, Var );
@@ -993,7 +1167,7 @@ Handle_CHANNEL( int Line, char *Var, char *Arg )
 } /* Handle_CHANNEL */
 
 
-static void
+static bool
 Validate_Config(bool Configtest, bool Rehash)
 {
 	/* Validate configuration settings. */
@@ -1001,6 +1175,7 @@ Validate_Config(bool Configtest, bool Rehash)
 #ifdef DEBUG
 	int i, servers, servers_once;
 #endif
+	bool config_valid = true;
 	char *ptr;
 
 	/* Validate configured server name, see RFC 2812 section 2.3.1 */
@@ -1019,6 +1194,7 @@ Validate_Config(bool Configtest, bool Rehash)
 
 	if (!Conf_ServerName[0]) {
 		/* No server name configured! */
+		config_valid = false;
 		Config_Error(LOG_ALERT,
 			     "No (valid) server name configured in \"%s\" (section 'Global': 'Name')!",
 			     NGIRCd_ConfFile);
@@ -1032,6 +1208,7 @@ Validate_Config(bool Configtest, bool Rehash)
 
 	if (Conf_ServerName[0] && !strchr(Conf_ServerName, '.')) {
 		/* No dot in server name! */
+		config_valid = false;
 		Config_Error(LOG_ALERT,
 			     "Invalid server name configured in \"%s\" (section 'Global': 'Name'): Dot missing!",
 			     NGIRCd_ConfFile);
@@ -1046,6 +1223,7 @@ Validate_Config(bool Configtest, bool Rehash)
 #ifdef STRICT_RFC
 	if (!Conf_ServerAdminMail[0]) {
 		/* No administrative contact configured! */
+		config_valid = false;
 		Config_Error(LOG_ALERT,
 			     "No administrator email address configured in \"%s\" ('AdminEMail')!",
 			     NGIRCd_ConfFile);
@@ -1078,6 +1256,8 @@ Validate_Config(bool Configtest, bool Rehash)
 	    "Configuration: Operators=%d, Servers=%d[%d], Channels=%d",
 	    Conf_Oper_Count, servers, servers_once, Conf_Channel_Count);
 #endif
+
+	return config_valid;
 } /* Validate_Config */
 
 
@@ -1144,6 +1324,7 @@ Init_Server_Struct( CONF_SERVER *Server )
 
 	Resolve_Init(&Server->res_stat);
 	Server->conn_id = NONE;
+	memset(&Server->bind_addr, 0, sizeof(&Server->bind_addr));
 } /* Init_Server_Struct */
 
 
