@@ -16,11 +16,12 @@
 
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: conn-func.c,v 1.10 2006/05/10 21:24:01 alex Exp $";
+static char UNUSED id[] = "$Id: conn-func.c,v 1.12 2008/03/11 14:05:27 alex Exp $";
 
 #include "imp.h"
 #include <assert.h>
-#include <log.h>
+#include <string.h>
+#include "log.h"
 
 #include "conn.h"
 #include "client.h"
@@ -39,6 +40,16 @@ Conn_UpdateIdle( CONN_ID Idx )
 }
 
 
+/*
+ * Get signon time of a connection.
+ */
+GLOBAL time_t
+Conn_GetSignon(CONN_ID Idx)
+{
+	assert(Idx > NONE);
+	return My_Connections[Idx].signon;
+}
+
 GLOBAL time_t
 Conn_GetIdle( CONN_ID Idx )
 {
