@@ -17,7 +17,7 @@
 #include "portab.h"
 #include "io.h"
 
-static char UNUSED id[] = "$Id: conn.c,v 1.198.2.6 2007/05/18 22:11:19 alex Exp $";
+static char UNUSED id[] = "$Id: conn.c,v 1.221 2008/02/26 22:04:17 fw Exp $";
 
 #include "imp.h"
 #include <assert.h>
@@ -41,12 +41,6 @@ static char UNUSED id[] = "$Id: conn.c,v 1.198.2.6 2007/05/18 22:11:19 alex Exp
 # include <netinet/ip.h>
 #endif
 
-#ifdef HAVE_ARPA_INET_H
-# include <arpa/inet.h>
-#else
-# define PF_INET AF_INET
-#endif
-
 #ifdef HAVE_STDINT_H
 # include <stdint.h>			/* e.g. for Mac OS X */
 #endif
@@ -92,10 +86,9 @@ static void Check_Connections PARAMS(( void ));
 static void Check_Servers PARAMS(( void ));
 static void Init_Conn_Struct PARAMS(( CONN_ID Idx ));
 static bool Init_Socket PARAMS(( int Sock ));
-static void New_Server PARAMS(( int Server ));
+static void New_Server PARAMS(( int Server, ng_ipaddr_t *dest ));
 static void Simple_Message PARAMS(( int Sock, const char *Msg ));
-static int Count_Connections PARAMS(( struct sockaddr_in addr ));
-static int NewListener PARAMS(( const UINT16 Port ));
+static int NewListener PARAMS(( const char *listen_addr, UINT16 Port ));
 
 static array My_Listeners;
 static array My_ConnArray;
@@ -150,10 +143,28 @@ cb_connserver(int sock, UNUSED short what)
  			    Conf_Server[Conf_GetServer(idx)].port,
  			    idx, strerror(err));
 
+		res = Conf_GetServer(idx);
+		assert(res >= 0);
+
 		Conn_Close(idx, "Can't connect!", NULL, false);
+
+		if (res < 0)
+			return;
+		if (ng_ipaddr_af(&Conf_Server[res].dst_addr[0])) {
+			/* more addresses to try... */
+			New_Server(res, &Conf_Server[res].dst_addr[0]);
+			/* connection to dst_addr[0] in progress, remove this address... */
+			Conf_Server[res].dst_addr[0] = Conf_Server[res].dst_addr[1];
+
+			memset(&Conf_Server[res].dst_addr[1], 0, sizeof(&Conf_Server[res].dst_addr[1]));
+		}
 		return;
 	}
 
+	res = Conf_GetServer(idx);
+	assert(res >= 0);
+	if (res >= 0) /* connect succeeded, remove all additional addresses */
+		memset(&Conf_Server[res].dst_addr, 0, sizeof(&Conf_Server[res].dst_addr));
 	Conn_OPTION_DEL( &My_Connections[idx], CONN_ISCONNECTING );
 	server_login(idx);
 }
@@ -260,25 +271,25 @@ Conn_Exit( void )
 } /* Conn_Exit */
 
 
-static int
-ports_initlisteners(array *a, void (*func)(int,short))
+static unsigned int
+ports_initlisteners(array *a, const char *listen_addr, void (*func)(int,short))
 {
-	int created = 0;
+	unsigned int created = 0;
 	size_t len;
 	int fd;
 	UINT16 *port;
 
 	len = array_length(a, sizeof (UINT16));
 	port = array_start(a);
-	while(len--) {
-		fd = NewListener( *port );
+	while (len--) {
+		fd = NewListener(listen_addr, *port);
 		if (fd < 0) {
 			port++;
 			continue;
 		}
 		if (!io_event_create( fd, IO_WANTREAD, func )) {
 			Log( LOG_ERR, "io_event_create(): Could not add listening fd %d (port %u): %s!",
-							fd, (unsigned int) *port, strerror(errno));
+						fd, (unsigned int) *port, strerror(errno));
 			close(fd);
 			port++;
 			continue;
@@ -286,25 +297,47 @@ ports_initlisteners(array *a, void (*func)(int,short))
 		created++;
 		port++;
 	}
-
 	return created;
 }
 
 
-GLOBAL int
+GLOBAL unsigned int
 Conn_InitListeners( void )
 {
 	/* Initialize ports on which the server should accept connections */
-
-	int created;
+	unsigned int created = 0;
+	char *copy, *listen_addr;
 
 	if (!io_library_init(CONNECTION_POOL)) {
 		Log(LOG_EMERG, "Cannot initialize IO routines: %s", strerror(errno));
 		return -1;
 	}
 
-	created = ports_initlisteners(&Conf_ListenPorts, cb_listen);
+	assert(Conf_ListenAddress);
 
+	/* can't use Conf_ListenAddress directly, see below */
+	copy = strdup(Conf_ListenAddress);
+	if (!copy) {
+		Log(LOG_CRIT, "Cannot copy %s: %s", Conf_ListenAddress, strerror(errno));
+		return 0;
+	}
+	listen_addr = strtok(copy, ",");
+
+	while (listen_addr) {
+		ngt_TrimStr(listen_addr);
+		if (*listen_addr)
+			created += ports_initlisteners(&Conf_ListenPorts, listen_addr, cb_listen);
+
+		listen_addr = strtok(NULL, ",");
+	}
+
+	/*
+	 * can't free() Conf_ListenAddress here. On /REHASH, if the config file
+	 * cannot be re-loaded, we'd end up with a NULL Conf_ListenAddress.
+	 * Instead, free() takes place in conf.c, before the config file
+	 * is being parsed.
+	 */
+	free(copy);
 	return created;
 } /* Conn_InitListeners */
 
@@ -333,52 +366,68 @@ Conn_ExitListeners( void )
 } /* Conn_ExitListeners */
 
 
+static bool
+InitSinaddrListenAddr(ng_ipaddr_t *addr, const char *listen_addrstr, UINT16 Port)
+{
+	bool ret;
+
+	ret = ng_ipaddr_init(addr, listen_addrstr, Port);
+	if (!ret) {
+		assert(listen_addrstr);
+		Log(LOG_CRIT, "Can't bind to [%s]:%u: can't convert ip address \"%s\"",
+						listen_addrstr, Port, listen_addrstr);
+	}
+	return ret;
+}
+
+
+static void
+set_v6_only(int af, int sock)
+{
+#if defined(IPV6_V6ONLY) && defined(WANT_IPV6)
+	int on = 1;
+
+	if (af != AF_INET6)
+		return;
+
+	if (setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on)))
+		Log(LOG_ERR, "Could not set IPV6_V6ONLY: %s", strerror(errno));
+#else
+	(void)af;
+	(void)sock;
+#endif
+}
+
+
 /* return new listening port file descriptor or -1 on failure */
 static int
-NewListener( const UINT16 Port )
+NewListener(const char *listen_addr, UINT16 Port)
 {
 	/* Create new listening socket on specified port */
-
-	struct sockaddr_in addr;
-	struct in_addr inaddr;
-	int sock;
+	ng_ipaddr_t addr;
+	int sock, af;
 #ifdef ZEROCONF
 	char name[CLIENT_ID_LEN], *info;
 #endif
+	if (!InitSinaddrListenAddr(&addr, listen_addr, Port))
+		return -1;
 
-	/* Server-"Listen"-Socket initialisieren */
-	memset( &addr, 0, sizeof( addr ));
-	memset( &inaddr, 0, sizeof( inaddr ));
-	addr.sin_family = (sa_family_t)AF_INET;
-	addr.sin_port = htons( Port );
-	if( Conf_ListenAddress[0] )
-	{
-#ifdef HAVE_INET_ATON
-		if( inet_aton( Conf_ListenAddress, &inaddr ) == 0 )
-#else
-		inaddr.s_addr = inet_addr( Conf_ListenAddress );
-		if( inaddr.s_addr == (unsigned)-1 )
-#endif
-		{
-			Log( LOG_CRIT, "Can't listen on %s:%u: can't convert ip address %s!",
-					Conf_ListenAddress, Port, Conf_ListenAddress );
-			return -1;
-		}
-	}
-	else inaddr.s_addr = htonl( INADDR_ANY );
-	addr.sin_addr = inaddr;
-
-	sock = socket( PF_INET, SOCK_STREAM, 0);
+	af = ng_ipaddr_af(&addr);
+	sock = socket(af, SOCK_STREAM, 0);
 	if( sock < 0 ) {
-		Log( LOG_CRIT, "Can't create socket: %s!", strerror( errno ));
+		Log(LOG_CRIT, "Can't create socket (af %d) : %s!", af, strerror(errno));
 		return -1;
 	}
 
-	if( ! Init_Socket( sock )) return -1;
+	set_v6_only(af, sock);
 
-	if (bind(sock, (struct sockaddr *)&addr, (socklen_t)sizeof(addr)) != 0) {
-		Log( LOG_CRIT, "Can't bind socket (port %d) : %s!", Port, strerror( errno ));
-		close( sock );
+	if (!Init_Socket(sock))
+		return -1;
+
+	if (bind(sock, (struct sockaddr *)&addr, ng_ipaddr_salen(&addr)) != 0) {
+		Log(LOG_CRIT, "Can't bind socket to address %s:%d - %s",
+			ng_ipaddr_tostr(&addr), Port, strerror(errno));
+		close(sock);
 		return -1;
 	}
 
@@ -395,8 +444,7 @@ NewListener( const UINT16 Port )
 		return -1;
 	}
 
-	if( Conf_ListenAddress[0]) Log( LOG_INFO, "Now listening on %s:%d (socket %d).", Conf_ListenAddress, Port, sock );
-	else Log( LOG_INFO, "Now listening on 0.0.0.0:%d (socket %d).", Port, sock );
+	Log(LOG_INFO, "Now listening on [%s]:%d (socket %d).", ng_ipaddr_tostr(&addr), Port, sock);
 
 #ifdef ZEROCONF
 	/* Get best server description text */
@@ -697,6 +745,7 @@ Conn_Close( CONN_ID Idx, char *LogMsg, char *FwdMsg, bool InformClient )
 	CLIENT *c;
 	char *txt;
 	double in_k, out_k;
+	UINT16 port;
 #ifdef ZLIB
 	double in_z_k, out_z_k;
 	int in_p, out_p;
@@ -724,9 +773,9 @@ Conn_Close( CONN_ID Idx, char *LogMsg, char *FwdMsg, bool InformClient )
 	if (! txt)
 		txt = "Reason unknown";
 
+	port = ng_ipaddr_getport(&My_Connections[Idx].addr);
 	Log(LOG_INFO, "Shutting down connection %d (%s) with %s:%d ...", Idx,
-	    LogMsg ? LogMsg : FwdMsg, My_Connections[Idx].host,
-	    ntohs(My_Connections[Idx].addr.sin_port));
+	    LogMsg ? LogMsg : FwdMsg, My_Connections[Idx].host, port);
 
 	/* Search client, if any */
 	c = Conn_GetClient( Idx );
@@ -766,7 +815,7 @@ Conn_Close( CONN_ID Idx, char *LogMsg, char *FwdMsg, bool InformClient )
 		Log(LOG_CRIT,
 		    "Error closing connection %d (socket %d) with %s:%d - %s! (ignored)",
 		    Idx, My_Connections[Idx].sock, My_Connections[Idx].host,
-		    ntohs(My_Connections[Idx].addr.sin_port), strerror(errno));
+		    port, strerror(errno));
 	}
 
 	/* Mark socket as invalid: */
@@ -795,8 +844,7 @@ Conn_Close( CONN_ID Idx, char *LogMsg, char *FwdMsg, bool InformClient )
 		out_p = (int)(( out_k * 100 ) / out_z_k );
 		Log(LOG_INFO,
 		    "Connection %d with %s:%d closed (in: %.1fk/%.1fk/%d%%, out: %.1fk/%.1fk/%d%%).",
-		    Idx, My_Connections[Idx].host,
-		    ntohs(My_Connections[Idx].addr.sin_port),
+		    Idx, My_Connections[Idx].host, port,
 		    in_k, in_z_k, in_p, out_k, out_z_k, out_p);
 	}
 	else
@@ -804,8 +852,7 @@ Conn_Close( CONN_ID Idx, char *LogMsg, char *FwdMsg, bool InformClient )
 	{
 		Log(LOG_INFO,
 		    "Connection %d with %s:%d closed (in: %.1fk, out: %.1fk).",
-		    Idx, My_Connections[Idx].host,
-		    ntohs(My_Connections[Idx].addr.sin_port),
+		    Idx, My_Connections[Idx].host, port,
 		    in_k, out_k);
 	}
 
@@ -928,6 +975,22 @@ Handle_Write( CONN_ID Idx )
 } /* Handle_Write */
 
 
+static int
+Count_Connections(ng_ipaddr_t *a)
+{
+	int i, cnt;
+
+	cnt = 0;
+	for (i = 0; i < Pool_Size; i++) {
+		if (My_Connections[i].sock <= NONE)
+			continue;
+		if (ng_ipaddr_ipequal(&My_Connections[i].addr, a))
+			cnt++;
+	}
+	return cnt;
+} /* Count_Connections */
+
+
 static int
 New_Connection( int Sock )
 {
@@ -937,14 +1000,16 @@ New_Connection( int Sock )
 #ifdef TCPWRAP
 	struct request_info req;
 #endif
-	struct sockaddr_in new_addr;
+	ng_ipaddr_t new_addr;
+	char ip_str[NG_INET_ADDRSTRLEN];
 	int new_sock, new_sock_len, new_Pool_Size;
 	CLIENT *c;
 	long cnt;
 
 	assert( Sock > NONE );
 	/* Connection auf Listen-Socket annehmen */
-	new_sock_len = (int)sizeof new_addr;
+	new_sock_len = (int)sizeof(new_addr);
+
 	new_sock = accept(Sock, (struct sockaddr *)&new_addr,
 			  (socklen_t *)&new_sock_len);
 	if (new_sock < 0) {
@@ -952,14 +1017,18 @@ New_Connection( int Sock )
 		return -1;
 	}
 
+	if (!ng_ipaddr_tostr_r(&new_addr, ip_str)) {
+		Log(LOG_CRIT, "fd %d: Can't convert IP address!", new_sock);
+		Simple_Message(new_sock, "ERROR :Internal Server Error");
+		close(new_sock);
+	}
+
 #ifdef TCPWRAP
 	/* Validate socket using TCP Wrappers */
 	request_init( &req, RQ_DAEMON, PACKAGE_NAME, RQ_FILE, new_sock, RQ_CLIENT_SIN, &new_addr, NULL );
 	fromhost(&req);
-	if( ! hosts_access( &req ))
-	{
-		/* Access denied! */
-		Log( deny_severity, "Refused connection from %s (by TCP Wrappers)!", inet_ntoa( new_addr.sin_addr ));
+	if (!hosts_access(&req)) {
+		Log (deny_severity, "Refused connection from %s (by TCP Wrappers)!", ip_str);
 		Simple_Message( new_sock, "ERROR :Connection refused" );
 		close( new_sock );
 		return -1;
@@ -969,13 +1038,12 @@ New_Connection( int Sock )
 	/* Socket initialisieren */
 	if (!Init_Socket( new_sock ))
 		return -1;
-	
+
 	/* Check IP-based connection limit */
-	cnt = Count_Connections( new_addr );
-	if(( Conf_MaxConnectionsIP > 0 ) && ( cnt >= Conf_MaxConnectionsIP ))
-	{
+	cnt = Count_Connections(&new_addr);
+	if ((Conf_MaxConnectionsIP > 0) && (cnt >= Conf_MaxConnectionsIP)) {
 		/* Access denied, too many connections from this IP address! */
-		Log( LOG_ERR, "Refused connection from %s: too may connections (%ld) from this IP address!", inet_ntoa( new_addr.sin_addr ), cnt);
+		Log( LOG_ERR, "Refused connection from %s: too may connections (%ld) from this IP address!", ip_str, cnt);
 		Simple_Message( new_sock, "ERROR :Connection refused, too many connections from your IP address!" );
 		close( new_sock );
 		return -1;
@@ -1017,7 +1085,7 @@ New_Connection( int Sock )
 		return -1;
 	}
 
-	c = Client_NewLocal( new_sock, inet_ntoa( new_addr.sin_addr ), CLIENT_UNKNOWN, false );
+	c = Client_NewLocal(new_sock, ip_str, CLIENT_UNKNOWN, false );
 	if( ! c ) {
 		Log(LOG_ALERT, "Can't accept connection: can't create client structure!");
 		Simple_Message(new_sock, "ERROR :Internal error");
@@ -1031,19 +1099,18 @@ New_Connection( int Sock )
 	My_Connections[new_sock].client = c;
 
 	Log( LOG_INFO, "Accepted connection %d from %s:%d on socket %d.", new_sock,
-			inet_ntoa( new_addr.sin_addr ), ntohs( new_addr.sin_port), Sock );
+			ip_str, ng_ipaddr_getport(&new_addr), Sock);
 
 	/* Hostnamen ermitteln */
-	strlcpy( My_Connections[new_sock].host, inet_ntoa( new_addr.sin_addr ),
-						sizeof( My_Connections[new_sock].host ));
+	strlcpy(My_Connections[new_sock].host, ip_str, sizeof(My_Connections[new_sock].host));
 
-	Client_SetHostname( c, My_Connections[new_sock].host );
+	Client_SetHostname(c, My_Connections[new_sock].host);
 
-	Resolve_Addr(&My_Connections[new_sock].res_stat, &new_addr,
-		My_Connections[new_sock].sock, cb_Read_Resolver_Result);
+	if (!Conf_NoDNS)
+		Resolve_Addr(&My_Connections[new_sock].res_stat, &new_addr,
+			My_Connections[new_sock].sock, cb_Read_Resolver_Result);
 
-	/* Penalty-Zeit setzen */
-	Conn_SetPenalty( new_sock, 4 );
+	Conn_SetPenalty(new_sock, 4);
 	return new_sock;
 } /* New_Connection */
 
@@ -1095,10 +1162,10 @@ Read_Request( CONN_ID Idx )
 
 	len = read(My_Connections[Idx].sock, readbuf, sizeof(readbuf));
 	if (len == 0) {
-		Log(LOG_INFO, "%s:%d (%s) is closing the connection ...",
-		    My_Connections[Idx].host,
-		    ntohs(My_Connections[Idx].addr.sin_port),
-		    inet_ntoa( My_Connections[Idx].addr.sin_addr));
+		Log(LOG_INFO, "%s:%u (%s) is closing the connection ...",
+				My_Connections[Idx].host,
+				(unsigned int) ng_ipaddr_getport(&My_Connections[Idx].addr),
+				ng_ipaddr_tostr(&My_Connections[Idx].addr));
 		Conn_Close(Idx,
 			   "Socket closed!", "Client closed connection",
 			   false);
@@ -1160,7 +1227,7 @@ Handle_Buffer( CONN_ID Idx )
 	/* Handle Data in Connections Read-Buffer.
 	 * Return true if a reuqest was handled, false otherwise (also returned on errors). */
 #ifndef STRICT_RFC
-	char *ptr1, *ptr2;
+	char *ptr1, *ptr2, *first_eol;
 #endif
 	char *ptr;
 	size_t len, delta;
@@ -1188,19 +1255,32 @@ Handle_Buffer( CONN_ID Idx )
 			return false;
 
 		/* A Complete Request end with CR+LF, see RFC 2812. */
+		delta = 2;
 		ptr = strstr( array_start(&My_Connections[Idx].rbuf), "\r\n" );
 
-		if( ptr ) delta = 2; /* complete request */
 #ifndef STRICT_RFC
-		else {
-			/* Check for non-RFC-compliant request (only CR or LF)? Unfortunately,
-			 * there are quite a few clients that do this (incl. "mIRC" :-( */
-			ptr1 = strchr( array_start(&My_Connections[Idx].rbuf), '\r' );
-			ptr2 = strchr( array_start(&My_Connections[Idx].rbuf), '\n' );
+		/* Check for non-RFC-compliant request (only CR or LF)?
+		 * Unfortunately, there are quite a few clients out there
+		 * that do this -- e. g. mIRC, BitchX, and Trillian :-( */
+		ptr1 = strchr(array_start(&My_Connections[Idx].rbuf), '\r');
+		ptr2 = strchr(array_start(&My_Connections[Idx].rbuf), '\n');
+		if (ptr) {
+			/* Check if there is a single CR or LF _before_ the
+			 * corerct CR+LF line terminator:  */
+			first_eol = ptr1 < ptr2 ? ptr1 : ptr2;
+			if (first_eol < ptr) {
+				/* Single CR or LF before CR+LF found */
+				ptr = first_eol;
+				delta = 1;
+			}
+		} else if (ptr1 || ptr2) {
+			/* No CR+LF terminated command found, but single
+			 * CR or LF found ... */
+			if (ptr1 && ptr2)
+				ptr = ptr1 < ptr2 ? ptr1 : ptr2;
+			else
+				ptr = ptr1 ? ptr1 : ptr2;
 			delta = 1;
-			if( ptr1 && ptr2 ) ptr = ptr1 > ptr2 ? ptr2 : ptr1;
-			else if( ptr1 ) ptr = ptr1;
-			else if( ptr2 ) ptr = ptr2;
 		}
 #endif
 
@@ -1333,7 +1413,7 @@ Check_Servers( void )
 		if( Conf_Server[i].group > NONE ) {
 			for (n = 0; n < MAX_SERVERS; n++) {
 				if (n == i) continue;
-				if ((Conf_Server[n].conn_id > NONE) &&
+				if ((Conf_Server[n].conn_id != NONE) &&
 					(Conf_Server[n].group == Conf_Server[i].group))
 						break;
 			}
@@ -1347,6 +1427,7 @@ Check_Servers( void )
 
 		/* Okay, try to connect now */
 		Conf_Server[i].lasttry = time_now;
+		Conf_Server[i].conn_id = SERVER_WAIT;
 		assert(Resolve_Getfd(&Conf_Server[i].res_stat) < 0);
 		Resolve_Name(&Conf_Server[i].res_stat, Conf_Server[i].host, cb_Connect_to_Server);
 	}
@@ -1354,54 +1435,50 @@ Check_Servers( void )
 
 
 static void
-New_Server( int Server )
+New_Server( int Server , ng_ipaddr_t *dest)
 {
 	/* Establish new server link */
-
-	struct sockaddr_in new_addr;
-	struct in_addr inaddr;
-	int res, new_sock;
+	char ip_str[NG_INET_ADDRSTRLEN];
+	int af_dest, res, new_sock;
 	CLIENT *c;
 
 	assert( Server > NONE );
 
-	Log( LOG_INFO, "Establishing connection to \"%s\", %s, port %d ... ", Conf_Server[Server].host,
-							Conf_Server[Server].ip, Conf_Server[Server].port );
+	if (!ng_ipaddr_tostr_r(dest, ip_str)) {
+		Log(LOG_WARNING, "New_Server: Could not convert IP to string");
+		return;
+	}
 
-#ifdef HAVE_INET_ATON
-	if( inet_aton( Conf_Server[Server].ip, &inaddr ) == 0 )
-#else
-	memset( &inaddr, 0, sizeof( inaddr ));
-	inaddr.s_addr = inet_addr( Conf_Server[Server].ip );
-	if( inaddr.s_addr == (unsigned)-1 )
-#endif
+	Log( LOG_INFO, "Establishing connection to \"%s\", %s, port %d ... ",
+			Conf_Server[Server].host, ip_str, Conf_Server[Server].port );
+
+	af_dest = ng_ipaddr_af(dest);
+	new_sock = socket(af_dest, SOCK_STREAM, 0);
+	if (new_sock < 0) {
+		Log( LOG_CRIT, "Can't create socket (af %d) : %s!", af_dest, strerror( errno ));
+		return;
+	}
+
+	if (!Init_Socket(new_sock))
+		return;
+
+	/* is a bind address configured? */
+	res = ng_ipaddr_af(&Conf_Server[Server].bind_addr);
+	/* if yes, bind now. If it fails, warn and let connect() pick a source address */
+	if (res && bind(new_sock, (struct sockaddr *) &Conf_Server[Server].bind_addr,
+				ng_ipaddr_salen(&Conf_Server[Server].bind_addr)))
 	{
-		Log( LOG_ERR, "Can't connect to \"%s\": can't convert ip address %s!",
-				Conf_Server[Server].host, Conf_Server[Server].ip );
-		return;
+		ng_ipaddr_tostr_r(&Conf_Server[Server].bind_addr, ip_str);
+		Log(LOG_WARNING, "Can't bind socket to %s: %s!", ip_str, strerror(errno));
 	}
-
-	memset( &new_addr, 0, sizeof( new_addr ));
-	new_addr.sin_family = (sa_family_t)AF_INET;
-	new_addr.sin_addr = inaddr;
-	new_addr.sin_port = htons( Conf_Server[Server].port );
-
-	new_sock = socket( PF_INET, SOCK_STREAM, 0 );
-	if ( new_sock < 0 ) {
-		Log( LOG_CRIT, "Can't create socket: %s!", strerror( errno ));
-		return;
-	}
-
-	if( ! Init_Socket( new_sock )) return;
-
-	res = connect(new_sock, (struct sockaddr *)&new_addr,
-			(socklen_t)sizeof(new_addr));
+	ng_ipaddr_setport(dest, Conf_Server[Server].port);
+	res = connect(new_sock, (struct sockaddr *) dest, ng_ipaddr_salen(dest));
 	if(( res != 0 ) && ( errno != EINPROGRESS )) {
 		Log( LOG_CRIT, "Can't connect socket: %s!", strerror( errno ));
 		close( new_sock );
 		return;
 	}
-	
+
 	if (!array_alloc(&My_ConnArray, sizeof(CONNECTION), (size_t)new_sock)) {
 		Log(LOG_ALERT,
 		    "Cannot allocate memory for server connection (socket %d)",
@@ -1416,8 +1493,9 @@ New_Server( int Server )
 
 	Init_Conn_Struct(new_sock);
 
-	c = Client_NewLocal( new_sock, inet_ntoa( new_addr.sin_addr ), CLIENT_UNKNOWNSERVER, false );
-	if( ! c ) {
+	ng_ipaddr_tostr_r(dest, ip_str);
+	c = Client_NewLocal(new_sock, ip_str, CLIENT_UNKNOWNSERVER, false);
+	if (!c) {
 		Log( LOG_ALERT, "Can't establish connection: can't create client structure!" );
 		close( new_sock );
 		return;
@@ -1429,7 +1507,7 @@ New_Server( int Server )
 	/* Register connection */
 	Conf_Server[Server].conn_id = new_sock;
 	My_Connections[new_sock].sock = new_sock;
-	My_Connections[new_sock].addr = new_addr;
+	My_Connections[new_sock].addr = *dest;
 	My_Connections[new_sock].client = c;
 	strlcpy( My_Connections[new_sock].host, Conf_Server[Server].host,
 				sizeof(My_Connections[new_sock].host ));
@@ -1448,14 +1526,17 @@ New_Server( int Server )
 } /* New_Server */
 
 
+/**
+ * Initialize connection structure.
+ */
 static void
-Init_Conn_Struct( CONN_ID Idx )
+Init_Conn_Struct(CONN_ID Idx)
 {
-	time_t now = time( NULL );
-	/* Connection-Struktur initialisieren */
+	time_t now = time(NULL);
 
-	memset( &My_Connections[Idx], 0, sizeof ( CONNECTION ));
+	memset(&My_Connections[Idx], 0, sizeof(CONNECTION));
 	My_Connections[Idx].sock = -1;
+	My_Connections[Idx].signon = now;
 	My_Connections[Idx].lastdata = now;
 	My_Connections[Idx].lastprivmsg = now;
 	Resolve_Init(&My_Connections[Idx].res_stat);
@@ -1505,7 +1586,9 @@ cb_Connect_to_Server(int fd, UNUSED short events)
 	/* Read result of resolver sub-process from pipe and start connection */
 	int i;
 	size_t len;
-	char readbuf[HOST_LEN + 1];
+	ng_ipaddr_t dest_addrs[4];	/* we can handle at most 3; but we read up to
+					   four so we can log the 'more than we can handle'
+					   condition */
 
 	LogDebug("Resolver: Got forward lookup callback on fd %d, events %d", fd, events);
 
@@ -1513,7 +1596,7 @@ cb_Connect_to_Server(int fd, UNUSED short events)
 		  if (Resolve_Getfd(&Conf_Server[i].res_stat) == fd )
 			  break;
 	}
-	
+
 	if( i >= MAX_SERVERS) {
 		/* Ops, no matching server found?! */
 		io_close( fd );
@@ -1522,16 +1605,28 @@ cb_Connect_to_Server(int fd, UNUSED short events)
 	}
 
 	/* Read result from pipe */
-	len = Resolve_Read(&Conf_Server[i].res_stat, readbuf, sizeof readbuf -1);
+	len = Resolve_Read(&Conf_Server[i].res_stat, dest_addrs, sizeof(dest_addrs));
 	if (len == 0)
 		return;
-	
-	readbuf[len] = '\0';
-	LogDebug("Got result from resolver: \"%s\" (%u bytes read).", readbuf, len);
-	strlcpy( Conf_Server[i].ip, readbuf, sizeof( Conf_Server[i].ip ));
 
+	assert((len % sizeof(ng_ipaddr_t)) == 0);
+
+	LogDebug("Got result from resolver: %u structs (%u bytes).", len/sizeof(ng_ipaddr_t), len);
+
+	memset(&Conf_Server[i].dst_addr, 0, sizeof(&Conf_Server[i].dst_addr));
+	if (len > sizeof(ng_ipaddr_t)) {
+		/* more than one address for this hostname, remember them
+		 * in case first address is unreachable/not available */
+		len -= sizeof(ng_ipaddr_t);
+		if (len > sizeof(&Conf_Server[i].dst_addr)) {
+			len = sizeof(&Conf_Server[i].dst_addr);
+			Log(LOG_NOTICE, "Notice: Resolver returned more IP Addresses for host than we can handle,"
+					" additional addresses dropped");
+		}
+		memcpy(&Conf_Server[i].dst_addr, &dest_addrs[1], len);
+	}
 	/* connect() */
-	New_Server(i);
+	New_Server(i, dest_addrs);
 } /* cb_Read_Forward_Lookup */
 
 
@@ -1598,10 +1693,10 @@ cb_Read_Resolver_Result( int r_fd, UNUSED short events )
 #ifdef IDENTAUTH
 		++identptr;
 		if (*identptr) {
-			Log( LOG_INFO, "IDENT lookup for connection %ld: \"%s\".", i, identptr);
-			Client_SetUser( c, identptr, true );
+			Log(LOG_INFO, "IDENT lookup for connection %d: \"%s\".", i, identptr);
+			Client_SetUser(c, identptr, true);
 		} else {
-			Log( LOG_INFO, "IDENT lookup for connection %ld: no result.", i );
+			Log(LOG_INFO, "IDENT lookup for connection %d: no result.", i);
 		}
 #endif
 	}
@@ -1630,19 +1725,6 @@ Simple_Message( int Sock, const char *Msg )
 } /* Simple_Error */
 
 
-static int
-Count_Connections( struct sockaddr_in addr_in )
-{
-	int i, cnt;
-	
-	cnt = 0;
-	for( i = 0; i < Pool_Size; i++ ) {
-		if(( My_Connections[i].sock > NONE ) && ( My_Connections[i].addr.sin_addr.s_addr == addr_in.sin_addr.s_addr )) cnt++;
-	}
-	return cnt;
-} /* Count_Connections */
-
-
 GLOBAL CLIENT *
 Conn_GetClient( CONN_ID Idx ) 
 {
