@@ -14,8 +14,6 @@
 
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: irc-mode.c,v 1.45.2.2 2007/04/03 20:23:31 fw Exp $";
-
 #include "imp.h"
 #include <assert.h>
 #include <stdio.h>
@@ -41,13 +39,10 @@ static char UNUSED id[] = "$Id: irc-mode.c,v 1.45.2.2 2007/04/03 20:23:31 fw Exp
 static bool Client_Mode PARAMS(( CLIENT *Client, REQUEST *Req, CLIENT *Origin, CLIENT *Target ));
 static bool Channel_Mode PARAMS(( CLIENT *Client, REQUEST *Req, CLIENT *Origin, CHANNEL *Channel ));
 
-static bool Add_Invite PARAMS(( CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, char *Pattern ));
-static bool Add_Ban PARAMS(( CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, char *Pattern ));
+static bool Add_Ban_Invite PARAMS((int what, CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, const char *Pattern));
+static bool Del_Ban_Invite PARAMS((int what, CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, const char *Pattern));
 
-static bool Del_Invite PARAMS(( CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, char *Pattern ));
-static bool Del_Ban PARAMS(( CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, char *Pattern ));
-
-static bool Send_ListChange PARAMS(( char *Mode, CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, char *Mask ));
+static bool Send_ListChange PARAMS(( char *Mode, CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, const char *Mask ));
 
 
 GLOBAL bool
@@ -164,6 +159,7 @@ Client_Mode( CLIENT *Client, REQUEST *Req, CLIENT *Origin, CLIENT *Target )
 		{
 			case 'i': /* Invisible */
 			case 's': /* Server messages */
+			case 'w': /* Wallops messages */
 				x[0] = *mode_ptr;
 				break;
 
@@ -241,123 +237,140 @@ client_exit:
 } /* Client_Mode */
 
 
+static bool
+Channel_Mode_Answer_Request(CLIENT *Origin, CHANNEL *Channel)
+{
+	char the_modes[COMMAND_LEN], the_args[COMMAND_LEN], argadd[CLIENT_PASS_LEN];
+	const char *mode_ptr;
+
+	/* Member or not? -- That's the question! */
+	if (!Channel_IsMemberOf(Channel, Origin))
+		return IRC_WriteStrClient(Origin, RPL_CHANNELMODEIS_MSG,
+			Client_ID(Origin), Channel_Name(Channel), Channel_Modes(Channel));
+
+	/* The sender is a member: generate extended reply */
+	strlcpy(the_modes, Channel_Modes(Channel), sizeof(the_modes));
+	mode_ptr = the_modes;
+	the_args[0] = '\0';
+
+	while(*mode_ptr) {
+		switch(*mode_ptr) {
+		case 'l':
+			snprintf(argadd, sizeof(argadd), " %lu", Channel_MaxUsers(Channel));
+			strlcat(the_args, argadd, sizeof(the_args));
+			break;
+		case 'k':
+			strlcat(the_args, " ", sizeof(the_args));
+			strlcat(the_args, Channel_Key(Channel), sizeof(the_args));
+			break;
+		}
+		mode_ptr++;
+	}
+	if (the_args[0])
+		strlcat(the_modes, the_args, sizeof(the_modes));
+
+	return IRC_WriteStrClient(Origin, RPL_CHANNELMODEIS_MSG,
+		Client_ID(Origin), Channel_Name(Channel), the_modes);
+}
+
+
 static bool
 Channel_Mode( CLIENT *Client, REQUEST *Req, CLIENT *Origin, CHANNEL *Channel )
 {
 	/* Handle channel and channel-user modes */
 
 	char the_modes[COMMAND_LEN], the_args[COMMAND_LEN], x[2], argadd[CLIENT_PASS_LEN], *mode_ptr;
-	bool ok, set, modeok = false, skiponce, use_servermode = false;
+	bool ok, set, modeok = true, skiponce, use_servermode = false, retval;
 	int mode_arg, arg_arg;
 	CLIENT *client;
 	long l;
 	size_t len;
 
 	/* Mode request: let's answer it :-) */
-	if( Req->argc == 1 )
-	{
-		/* Member or not? -- That's the question! */
-		if( ! Channel_IsMemberOf( Channel, Origin )) return IRC_WriteStrClient( Origin, RPL_CHANNELMODEIS_MSG, Client_ID( Origin ), Channel_Name( Channel ), Channel_Modes( Channel ));
-
-		/* The sender is a member: generate extended reply */
-		strlcpy( the_modes, Channel_Modes( Channel ), sizeof( the_modes ));
-		mode_ptr = the_modes;
-		the_args[0] = '\0';
-		while( *mode_ptr )
-		{
-			switch( *mode_ptr )
-			{
-				case 'l':
-					snprintf( argadd, sizeof( argadd ), " %lu", Channel_MaxUsers( Channel ));
-					strlcat( the_args, argadd, sizeof( the_args ));
-					break;
-				case 'k':
-					strlcat( the_args, " ", sizeof( the_args ));
-					strlcat( the_args, Channel_Key( Channel ), sizeof( the_args ));
-					break;
-			}
-			mode_ptr++;
-		}
-		if( the_args[0] ) strlcat( the_modes, the_args, sizeof( the_modes ));
-
-		return IRC_WriteStrClient( Origin, RPL_CHANNELMODEIS_MSG, Client_ID( Origin ), Channel_Name( Channel ), the_modes );
-	}
+	if (Req->argc <= 1)
+		return Channel_Mode_Answer_Request(Origin, Channel);
 
 	/* Is the user allowed to change modes? */
-	if( Client_Type( Client ) == CLIENT_USER )
-	{
+	if (Client_Type(Client) == CLIENT_USER) {
 		/* Is the originating user on that channel? */
-		if( ! Channel_IsMemberOf( Channel, Origin )) return IRC_WriteStrClient( Origin, ERR_NOTONCHANNEL_MSG, Client_ID( Origin ), Channel_Name( Channel ));
-
-		/* Is he channel operator? */
-		if( strchr( Channel_UserModes( Channel, Origin ), 'o' )) modeok = true;
-		else if( Conf_OperCanMode )
-		{
+		if (!Channel_IsMemberOf(Channel, Origin))
+			return IRC_WriteStrClient(Origin, ERR_NOTONCHANNEL_MSG,
+				Client_ID(Origin), Channel_Name(Channel));
+		modeok = false;
+		/* channel operator? */
+		if (strchr(Channel_UserModes(Channel, Origin), 'o'))
+			modeok = true;
+		else if(Conf_OperCanMode) {
 			/* IRC-Operators can use MODE as well */
-			if( Client_OperByMe( Origin )) {
+			if (Client_OperByMe(Origin)) {
 				modeok = true;
-				if ( Conf_OperServerMode ) use_servermode = true; /* Change Origin to Server */
+				if (Conf_OperServerMode)
+					use_servermode = true; /* Change Origin to Server */
 			}
 		}
 	}
-	else modeok = true;
 
 	mode_arg = 1;
 	mode_ptr = Req->argv[mode_arg];
-	if( Req->argc > mode_arg + 1 ) arg_arg = mode_arg + 1;
-	else arg_arg = -1;
+	if (Req->argc > mode_arg + 1)
+		arg_arg = mode_arg + 1;
+	else
+		arg_arg = -1;
 
 	/* Initial state: set or unset modes? */
 	skiponce = false;
-	if( *mode_ptr == '-' ) set = false;
-	else if( *mode_ptr == '+' ) set = true;
-	else set = skiponce = true;
+	switch (*mode_ptr) {
+	case '-': set = false; break;
+	case '+': set = true; break;
+	default:
+		set = true;
+		skiponce = true;
+	}
 
 	/* Prepare reply string */
-	if( set ) strcpy( the_modes, "+" );
-	else strcpy( the_modes, "-" );
-	strcpy( the_args, " " );
+	strcpy(the_modes, set ? "+" : "-");
+	the_args[0] = '\0';
 
 	x[1] = '\0';
 	ok = CONNECTED;
-	while( mode_ptr )
-	{
-		if( ! skiponce ) mode_ptr++;
-		if( ! *mode_ptr )
-		{
+	while (mode_ptr) {
+		if (! skiponce)
+			mode_ptr++;
+		if (!*mode_ptr) {
 			/* Try next argument if there's any */
-			if( arg_arg > mode_arg ) mode_arg = arg_arg;
-			else mode_arg++;
-			if( mode_arg < Req->argc ) mode_ptr = Req->argv[mode_arg];
-			else break;
-			if( Req->argc > mode_arg + 1 ) arg_arg = mode_arg + 1;
-			else arg_arg = -1;
+			if (arg_arg > mode_arg)
+				mode_arg = arg_arg;
+			else
+				mode_arg++;
+
+			if (mode_arg >= Req->argc)
+				break;
+			mode_ptr = Req->argv[mode_arg];
+
+			if (Req->argc > mode_arg + 1)
+				arg_arg = mode_arg + 1;
+			else
+				arg_arg = -1;
 		}
 		skiponce = false;
 
-		switch( *mode_ptr )
-		{
-			case '+':
-			case '-':
-				if((( *mode_ptr == '+' ) && ( ! set )) || (( *mode_ptr == '-' ) && ( set )))
-				{
-					/* Action modifier ("+"/"-") must be changed ... */
-					len = strlen( the_modes ) - 1;
-					if(( the_modes[len] == '+' ) || ( the_modes[len] == '-' ))
-					{
-						/* Adjust last action modifier in result */
-						the_modes[len] = *mode_ptr;
-					}
-					else
-					{
-						/* Append modifier character to result string */
-						x[0] = *mode_ptr;
-						strlcat( the_modes, x, sizeof( the_modes ));
-					}
-					if( *mode_ptr == '+' ) set = true;
-					else set = false;
+		switch (*mode_ptr) {
+		case '+':
+		case '-':
+			if (((*mode_ptr == '+') && !set) || ((*mode_ptr == '-') && set)) {
+				/* Action modifier ("+"/"-") must be changed ... */
+				len = strlen( the_modes ) - 1;
+				if ((the_modes[len] == '+') || (the_modes[len] == '-')) {
+					/* Adjust last action modifier in result */
+					the_modes[len] = *mode_ptr;
+				} else {
+					/* Append modifier character to result string */
+					x[0] = *mode_ptr;
+					strlcat(the_modes, x, sizeof(the_modes));
 				}
-				continue;
+				set = *mode_ptr == '+';
+			}
+			continue;
 		}
 
 		/* Are there arguments left? */
@@ -367,240 +380,194 @@ Channel_Mode( CLIENT *Client, REQUEST *Req, CLIENT *Origin, CHANNEL *Channel )
 		x[0] = '\0';
 		argadd[0] = '\0';
 		client = NULL;
-		switch( *mode_ptr )
-		{
-			/* --- Channel modes --- */
-
-			case 'i': /* Invite only */
-			case 'm': /* Moderated */
-			case 'n': /* Only members can write */
-			case 's': /* Secret channel */
-			case 't': /* Topic locked */
-				if( modeok ) x[0] = *mode_ptr;
-				else ok = IRC_WriteStrClient( Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID( Origin ), Channel_Name( Channel ));
+		switch (*mode_ptr) {
+		/* --- Channel modes --- */
+		case 'i': /* Invite only */
+		case 'm': /* Moderated */
+		case 'n': /* Only members can write */
+		case 's': /* Secret channel */
+		case 't': /* Topic locked */
+			if (modeok)
+				x[0] = *mode_ptr;
+			else
+				ok = IRC_WriteStrClient(Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID(Origin), Channel_Name(Channel));
+			break;
+		case 'k': /* Channel key */
+			if (! set) {
+				if (modeok)
+					x[0] = *mode_ptr;
+				else
+					ok = IRC_WriteStrClient(Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID(Origin), Channel_Name(Channel));
 				break;
-
-			case 'k': /* Channel key */
-				if( ! set )
-				{
-					if( modeok ) x[0] = *mode_ptr;
-					else ok = IRC_WriteStrClient( Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID( Origin ), Channel_Name( Channel ));
-					break;
+			}
+			if (arg_arg > mode_arg) {
+				if (modeok) {
+					Channel_ModeDel(Channel, 'k');
+					Channel_SetKey(Channel, Req->argv[arg_arg]);
+					strlcpy(argadd, Channel_Key(Channel), sizeof(argadd));
+					x[0] = *mode_ptr;
+				} else {
+					ok = IRC_WriteStrClient(Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID(Origin), Channel_Name(Channel));
 				}
-				if( arg_arg > mode_arg )
-				{
-					if( modeok )
-					{
-						Channel_ModeDel( Channel, 'k' );
-						Channel_SetKey( Channel, Req->argv[arg_arg] );
-						strlcpy( argadd, Channel_Key( Channel ), sizeof( argadd ));
+				Req->argv[arg_arg][0] = '\0';
+				arg_arg++;
+			} else {
+				ok = IRC_WriteStrClient(Origin, ERR_NEEDMOREPARAMS_MSG, Client_ID(Origin), Req->command);
+			}
+			break;
+		case 'l': /* Member limit */
+			if (!set) {
+				if (modeok)
+					x[0] = *mode_ptr;
+				else
+					ok = IRC_WriteStrClient(Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID(Origin), Channel_Name(Channel));
+				break;
+			}
+			if (arg_arg > mode_arg) {
+				if (modeok) {
+					l = atol(Req->argv[arg_arg]);
+					if (l > 0 && l < 0xFFFF) {
+						Channel_ModeDel(Channel, 'l');
+						Channel_SetMaxUsers(Channel, l);
+						snprintf(argadd, sizeof(argadd), "%ld", l);
 						x[0] = *mode_ptr;
 					}
-					else ok = IRC_WriteStrClient( Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID( Origin ), Channel_Name( Channel ));
-					Req->argv[arg_arg][0] = '\0';
-					arg_arg++;
+				} else {
+					ok = IRC_WriteStrClient(Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID(Origin), Channel_Name(Channel));
 				}
-				else ok = IRC_WriteStrClient( Origin, ERR_NEEDMOREPARAMS_MSG, Client_ID( Origin ), Req->command );
-				break;
-
-			case 'l': /* Member limit */
-				if( ! set )
-				{
-					if( modeok ) x[0] = *mode_ptr;
-					else ok = IRC_WriteStrClient( Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID( Origin ), Channel_Name( Channel ));
-					break;
+				Req->argv[arg_arg][0] = '\0';
+				arg_arg++;
+			} else {
+				ok = IRC_WriteStrClient(Origin, ERR_NEEDMOREPARAMS_MSG, Client_ID(Origin), Req->command);
+			}
+			break;
+		case 'P': /* Persistent channel */
+			if (modeok) {
+				/* Only IRC operators are allowed to
+				 * set the 'P' channel mode! */
+				if (set && !(Client_OperByMe(Client) || Client_Type(Client) == CLIENT_SERVER))
+					ok = IRC_WriteStrClient(Origin, ERR_NOPRIVILEGES_MSG, Client_ID(Origin));
+				else
+						x[0] = 'P';
+			} else
+				ok = IRC_WriteStrClient(Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID(Origin), Channel_Name(Channel));
+			break;
+		/* --- Channel user modes --- */
+		case 'o': /* Channel operator */
+		case 'v': /* Voice */
+			if (arg_arg > mode_arg) {
+				if (modeok) {
+					client = Client_Search(Req->argv[arg_arg]);
+					if (client)
+						x[0] = *mode_ptr;
+					else
+						ok = IRC_WriteStrClient(Client, ERR_NOSUCHNICK_MSG, Client_ID(Client), Req->argv[arg_arg]);
+				} else {
+					ok = IRC_WriteStrClient(Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID(Origin), Channel_Name(Channel));
 				}
-				if( arg_arg > mode_arg )
-				{
-					if( modeok )
-					{
-						l = atol( Req->argv[arg_arg] );
-						if( l > 0 && l < 0xFFFF )
-						{
-							Channel_ModeDel( Channel, 'l' );
-							Channel_SetMaxUsers( Channel, l );
-							snprintf( argadd, sizeof( argadd ), "%ld", l );
-							x[0] = *mode_ptr;
-						}
-					}
-					else ok = IRC_WriteStrClient( Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID( Origin ), Channel_Name( Channel ));
-					Req->argv[arg_arg][0] = '\0';
-					arg_arg++;
+				Req->argv[arg_arg][0] = '\0';
+				arg_arg++;
+			} else {
+				ok = IRC_WriteStrClient(Origin, ERR_NEEDMOREPARAMS_MSG, Client_ID(Origin), Req->command);
+			}
+			break;
+		/* --- Channel lists --- */
+		case 'I': /* Invite lists */
+		case 'b': /* Ban lists */
+			if (arg_arg > mode_arg) {
+				/* modify list */
+				if (modeok) {
+					ok = set ? Add_Ban_Invite(*mode_ptr, Origin, Client, Channel, Req->argv[arg_arg])
+						 : Del_Ban_Invite(*mode_ptr, Origin, Client, Channel, Req->argv[arg_arg]);
+				} else {
+					ok = IRC_WriteStrClient(Origin, ERR_CHANOPRIVSNEEDED_MSG,
+							Client_ID(Origin), Channel_Name(Channel));
 				}
-				else ok = IRC_WriteStrClient( Origin, ERR_NEEDMOREPARAMS_MSG, Client_ID( Origin ), Req->command );
-				break;
+				Req->argv[arg_arg][0] = '\0';
+				arg_arg++;
+			} else {
+				if (*mode_ptr == 'I')
+					Channel_ShowInvites(Origin, Channel);
+				else
+					Channel_ShowBans(Origin, Channel);
+			}
+			break;
+		default:
+			Log(LOG_DEBUG, "Unknown mode \"%c%c\" from \"%s\" on %s!?",
+				set ? '+' : '-', *mode_ptr, Client_ID(Origin), Channel_Name(Channel));
+			if (Client_Type(Client) != CLIENT_SERVER)
+				ok = IRC_WriteStrClient(Origin, ERR_UMODEUNKNOWNFLAG2_MSG, Client_ID(Origin), set ? '+' : '-', *mode_ptr);
+			x[0] = '\0';
+			goto chan_exit;
+		} /* switch() */
 
-			case 'P': /* Persistent channel */
-				if( modeok )
-				{
-					if( set && ( ! Client_OperByMe( Client )))
-					{
-						/* Only IRC operators are allowed to set P mode */
-						ok = IRC_WriteStrClient( Origin, ERR_NOPRIVILEGES_MSG, Client_ID( Origin ));
-					}
-					else x[0] = 'P';
-				}
-				else ok = IRC_WriteStrClient( Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID( Origin ), Channel_Name( Channel ));
-				break;
-
-			/* --- Channel user modes --- */
-
-			case 'o': /* Channel operator */
-			case 'v': /* Voice */
-				if( arg_arg > mode_arg )
-				{
-					if( modeok )
-					{
-						client = Client_Search( Req->argv[arg_arg] );
-						if( client ) x[0] = *mode_ptr;
-						else ok = IRC_WriteStrClient( Client, ERR_NOSUCHNICK_MSG, Client_ID( Client ), Req->argv[arg_arg] );
-					}
-					else ok = IRC_WriteStrClient( Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID( Origin ), Channel_Name( Channel ));
-					Req->argv[arg_arg][0] = '\0';
-					arg_arg++;
-				}
-				else ok = IRC_WriteStrClient( Origin, ERR_NEEDMOREPARAMS_MSG, Client_ID( Origin ), Req->command );
-				break;
-
-			/* --- Channel lists --- */
-
-			case 'I': /* Invite lists */
-				if( arg_arg > mode_arg )
-				{
-					/* modify list */
-					if( modeok )
-					{
-						if( set ) Add_Invite( Origin, Client, Channel, Req->argv[arg_arg] );
-						else Del_Invite( Origin, Client, Channel, Req->argv[arg_arg] );
-					}
-					else ok = IRC_WriteStrClient( Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID( Origin ), Channel_Name( Channel ));
-					Req->argv[arg_arg][0] = '\0';
-					arg_arg++;
-				}
-				else Channel_ShowInvites( Origin, Channel );
-				break;
-
-			case 'b': /* Ban lists */
-				if( arg_arg > mode_arg )
-				{
-					/* modify list */
-					if( modeok )
-					{
-						if( set ) Add_Ban( Origin, Client, Channel, Req->argv[arg_arg] );
-						else Del_Ban( Origin, Client, Channel, Req->argv[arg_arg] );
-					}
-					else ok = IRC_WriteStrClient( Origin, ERR_CHANOPRIVSNEEDED_MSG, Client_ID( Origin ), Channel_Name( Channel ));
-					Req->argv[arg_arg][0] = '\0';
-					arg_arg++;
-				}
-				else Channel_ShowBans( Origin, Channel );
-				break;
-
-			default:
-				Log( LOG_DEBUG, "Unknown mode \"%c%c\" from \"%s\" on %s!?", set ? '+' : '-', *mode_ptr, Client_ID( Origin ), Channel_Name( Channel ));
-				if( Client_Type( Client ) != CLIENT_SERVER ) ok = IRC_WriteStrClient( Origin, ERR_UMODEUNKNOWNFLAG2_MSG, Client_ID( Origin ), set ? '+' : '-', *mode_ptr );
-				x[0] = '\0';
-				goto chan_exit;
-		}
-		if( ! ok ) break;
+		if (!ok)
+			break;
 
 		/* Is there a valid mode change? */
-		if( ! x[0] ) continue;
+		if (!x[0])
+			continue;
 
 		/* Validate target client */
-		if( client && ( ! Channel_IsMemberOf( Channel, client )))
-		{
-			if( ! IRC_WriteStrClient( Origin, ERR_USERNOTINCHANNEL_MSG, Client_ID( Origin ), Client_ID( client ), Channel_Name( Channel ))) break;
+		if (client && (!Channel_IsMemberOf(Channel, client))) {
+			if (!IRC_WriteStrClient(Origin, ERR_USERNOTINCHANNEL_MSG,
+				Client_ID(Origin), Client_ID(client), Channel_Name(Channel)))
+					break;
+
 			continue;
 		}
 
-		if( set )
-		{
-			/* Set mode */
-			if( client )
-			{
-				/* Channel-User-Mode */
-				if( Channel_UserModeAdd( Channel, client, x[0] ))
-				{
-					strlcat( the_args, Client_ID( client ), sizeof( the_args ));
-					strlcat( the_args, " ", sizeof( the_args ));
-					strlcat( the_modes, x, sizeof( the_modes ));
-					Log( LOG_DEBUG, "User \"%s\": Mode change on %s, now \"%s\"", Client_Mask( client ), Channel_Name( Channel ), Channel_UserModes( Channel, client ));
-				}
+		if (client) {
+			/* Channel-User-Mode */
+			retval = set ? Channel_UserModeAdd(Channel, client, x[0]) : Channel_UserModeDel(Channel, client, x[0]);
+			if (retval) {
+				strlcat(the_args, " ", sizeof(the_args));
+				strlcat(the_args, Client_ID(client), sizeof(the_args));
+				strlcat(the_modes, x, sizeof(the_modes));
+				Log(LOG_DEBUG, "User \"%s\": Mode change on %s, now \"%s\"",
+					Client_Mask(client), Channel_Name(Channel), Channel_UserModes(Channel, client));
 			}
-			else
-			{
-				/* Channel-Mode */
-				if( Channel_ModeAdd( Channel, x[0] ))
-				{
-					strlcat( the_modes, x, sizeof( the_modes ));
-					Log( LOG_DEBUG, "Channel %s: Mode change, now \"%s\".", Channel_Name( Channel ), Channel_Modes( Channel ));
-				}
-			}
-		}
-		else
-		{
-			/* Unset mode */
-			if( client )
-			{
-				/* Channel-User-Mode */
-				if( Channel_UserModeDel( Channel, client, x[0] ))
-				{
-					strlcat( the_args, Client_ID( client ), sizeof( the_args ));
-					strlcat( the_args, " ", sizeof( the_args ));
-					strlcat( the_modes, x, sizeof( the_modes ));
-					Log( LOG_DEBUG, "User \"%s\": Mode change on %s, now \"%s\"", Client_Mask( client ), Channel_Name( Channel ), Channel_UserModes( Channel, client ));
-				}
-			}
-			else
-			{
-				/* Channel-Mode */
-				if( Channel_ModeDel( Channel, x[0] ))
-				{
-					strlcat( the_modes, x, sizeof( the_modes ));
-					Log( LOG_DEBUG, "Channel %s: Mode change, now \"%s\".", Channel_Name( Channel ), Channel_Modes( Channel ));
-				}
+		} else {
+			/* Channel-Mode */
+			retval = set ? Channel_ModeAdd(Channel, x[0]) : Channel_ModeDel(Channel, x[0]);
+			if (retval) {
+				strlcat(the_modes, x, sizeof(the_modes));
+				Log(LOG_DEBUG, "Channel %s: Mode change, now \"%s\".", Channel_Name(Channel), Channel_Modes(Channel));
 			}
 		}
 
 		/* Are there additional arguments to add? */
-		if( argadd[0] )
-		{
-			len = strlen( the_args ) - 1;
-			if( the_args[len] != ' ' ) strlcat( the_args, " ", sizeof( the_args ));
-			strlcat( the_args, argadd, sizeof( the_args ));
+		if (argadd[0]) {
+			strlcat(the_args, " ", sizeof(the_args));
+			strlcat(the_args, argadd, sizeof(the_args));
 		}
 	}
 chan_exit:
-
 	/* Are there changed modes? */
-	if( the_modes[1] )
-	{
+	if (the_modes[1]) {
 		/* Clean up mode string */
-		len = strlen( the_modes ) - 1;
-		if(( the_modes[len] == '+' ) || ( the_modes[len] == '-' )) the_modes[len] = '\0';
+		len = strlen(the_modes) - 1;
+		if ((the_modes[len] == '+') || (the_modes[len] == '-'))
+			the_modes[len] = '\0';
 
-		/* Clean up argument string if there are none */
-		if( ! the_args[1] ) the_args[0] = '\0';
-
-		if( Client_Type( Client ) == CLIENT_SERVER )
-		{
+		if (Client_Type(Client) == CLIENT_SERVER) {
 			/* Forward mode changes to channel users and other servers */
-			IRC_WriteStrServersPrefix( Client, Origin, "MODE %s %s%s", Channel_Name( Channel ), the_modes, the_args );
-			IRC_WriteStrChannelPrefix( Client, Channel, Origin, false, "MODE %s %s%s", Channel_Name( Channel ), the_modes, the_args );
-		}
-		else
-		{
-			if ( use_servermode ) Origin = Client_ThisServer();
-
+			IRC_WriteStrServersPrefix(Client, Origin, "MODE %s %s%s", Channel_Name( Channel ), the_modes, the_args);
+			IRC_WriteStrChannelPrefix(Client, Channel, Origin, false, "MODE %s %s%s", Channel_Name(Channel), the_modes, the_args);
+		} else {
+			if (use_servermode)
+				Origin = Client_ThisServer();
 			/* Send reply to client and inform other servers and channel users */
-			ok = IRC_WriteStrClientPrefix( Client, Origin, "MODE %s %s%s", Channel_Name( Channel ), the_modes, the_args );
-			IRC_WriteStrServersPrefix( Client, Origin, "MODE %s %s%s", Channel_Name( Channel ), the_modes, the_args );
-			IRC_WriteStrChannelPrefix( Client, Channel, Origin, false, "MODE %s %s%s", Channel_Name( Channel ), the_modes, the_args );
+			ok = IRC_WriteStrClientPrefix(Client, Origin, "MODE %s %s%s",
+					Channel_Name(Channel), the_modes, the_args);
+			IRC_WriteStrServersPrefix(Client, Origin, "MODE %s %s%s",
+					Channel_Name(Channel), the_modes, the_args);
+			IRC_WriteStrChannelPrefix(Client, Channel, Origin, false, "MODE %s %s%s",
+						Channel_Name(Channel), the_modes, the_args);
 		}
 	}
 
-	IRC_SetPenalty( Client, 1 );
+	IRC_SetPenalty(Client, 1);
 	return CONNECTED;
 } /* Channel_Mode */
 
@@ -633,85 +600,64 @@ IRC_AWAY( CLIENT *Client, REQUEST *Req )
 
 
 static bool
-Add_Invite( CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, char *Pattern )
+Add_Ban_Invite(int what, CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, const char *Pattern)
 {
-	char *mask;
+	const char *mask;
 	bool already;
+	bool ret;
 
 	assert( Client != NULL );
 	assert( Channel != NULL );
 	assert( Pattern != NULL );
+	assert(what == 'I' || what == 'b');
 
-	mask = Lists_MakeMask( Pattern );
+	mask = Lists_MakeMask(Pattern);
 
-	already = Lists_CheckDupeMask(Channel_GetListInvites(Channel), mask );
+	already = Lists_CheckDupeMask(Channel_GetListInvites(Channel), mask);
 	if (!already) {
-		if( ! Channel_AddInvite(Channel, mask, false ))
+		if (what == 'I')
+			ret = Channel_AddInvite(Channel, mask, false);
+		else
+			ret = Channel_AddBan(Channel, mask);
+		if (!ret)
 			return CONNECTED;
 	}
-	if ( already && ( Client_Type( Prefix ) == CLIENT_SERVER ))
+	if (already && (Client_Type(Prefix) == CLIENT_SERVER))
 		return CONNECTED;
 
-	return Send_ListChange( "+I", Prefix, Client, Channel, mask );
-} /* Add_Invite */
+	if (what == 'I')
+		return Send_ListChange("+I", Prefix, Client, Channel, mask);
+	return Send_ListChange("+b", Prefix, Client, Channel, mask);
+}
 
 
 static bool
-Add_Ban( CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, char *Pattern )
+Del_Ban_Invite(int what, CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, const char *Pattern)
 {
-	char *mask;
-	bool already;
+	const char *mask;
+	struct list_head *list;
 
 	assert( Client != NULL );
 	assert( Channel != NULL );
 	assert( Pattern != NULL );
+	assert(what == 'I' || what == 'b');
 
 	mask = Lists_MakeMask( Pattern );
 
-	already = Lists_CheckDupeMask(Channel_GetListBans(Channel), mask );
-	if (!already) {
-		if( ! Channel_AddBan(Channel, mask))
-			return CONNECTED;
-	}
-	if ( already && ( Client_Type( Prefix ) == CLIENT_SERVER ))
-		return CONNECTED;
+	if (what == 'I')
+		list = Channel_GetListInvites(Channel);
+	else
+		list = Channel_GetListBans(Channel);
 
-	return Send_ListChange( "+b", Prefix, Client, Channel, mask );
-} /* Add_Ban */
-
-
-static bool
-Del_Invite( CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, char *Pattern )
-{
-	char *mask;
-
-	assert( Client != NULL );
-	assert( Channel != NULL );
-	assert( Pattern != NULL );
-
-	mask = Lists_MakeMask( Pattern );
-	Lists_Del(Channel_GetListInvites(Channel), mask);
-	return Send_ListChange( "-I", Prefix, Client, Channel, mask );
-} /* Del_Invite */
-
-
-static bool
-Del_Ban( CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, char *Pattern )
-{
-	char *mask;
-
-	assert( Client != NULL );
-	assert( Channel != NULL );
-	assert( Pattern != NULL );
-
-	mask = Lists_MakeMask( Pattern );
-	Lists_Del(Channel_GetListBans(Channel), mask);
+	Lists_Del(list, mask);
+	if (what == 'I')
+		return Send_ListChange( "-I", Prefix, Client, Channel, mask );
 	return Send_ListChange( "-b", Prefix, Client, Channel, mask );
-} /* Del_Ban */
+}
 
 
 static bool
-Send_ListChange( char *Mode, CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, char *Mask )
+Send_ListChange( char *Mode, CLIENT *Prefix, CLIENT *Client, CHANNEL *Channel, const char *Mask )
 {
 	/* Bestaetigung an Client schicken & andere Server sowie Channel-User informieren */
 
