@@ -1,6 +1,6 @@
 /*
  * ngIRCd -- The Next Generation IRC Daemon
- * Copyright (c)2001-2007 Alexander Barton (alex@barton.de).
+ * Copyright (c)2001-2008 Alexander Barton (alex@barton.de).
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,8 +12,6 @@
 
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: ngircd.c,v 1.113.2.2 2007/05/05 20:25:47 alex Exp $";
-
 /**
  * @file
  * The main program, including the C function main() which is called
@@ -42,7 +40,6 @@ static char UNUSED id[] = "$Id: ngircd.c,v 1.113.2.2 2007/05/05 20:25:47 alex Ex
 #include "client.h"
 #include "channel.h"
 #include "conf.h"
-#include "cvs-version.h"
 #include "lists.h"
 #include "log.h"
 #include "parse.h"
@@ -169,14 +166,12 @@ main( int argc, const char *argv[] )
 			{
 				ok = false;
 #ifdef DEBUG
-				if( argv[i][n] == 'd' )
-				{
+				if (argv[i][n] == 'd') {
 					NGIRCd_Debug = true;
 					ok = true;
 				}
 #endif
-				if( argv[i][n] == 'f' )
-				{
+				if (argv[i][n] == 'f') {
 					if(( ! argv[i][n + 1] ) && ( i + 1 < argc ))
 					{
 						/* Ok, next character is a blank */
@@ -188,31 +183,38 @@ main( int argc, const char *argv[] )
 						ok = true;
 					}
 				}
-				if( argv[i][n] == 'n' )
-				{
+
+				if (argv[i][n] == 'h') {
+					Show_Version();
+					puts(""); Show_Help(); puts("");
+					exit(1);
+				}
+
+				if (argv[i][n] == 'n') {
 					NGIRCd_NoDaemon = true;
 					ok = true;
 				}
-				if( argv[i][n] == 'p' )
-				{
+				if (argv[i][n] == 'p') {
 					NGIRCd_Passive = true;
 					ok = true;
 				}
 #ifdef SNIFFER
-				if( argv[i][n] == 's' )
-				{
+				if (argv[i][n] == 's') {
 					NGIRCd_Sniffer = true;
 					ok = true;
 				}
 #endif
-				if( argv[i][n] == 't' )
-				{
+				if (argv[i][n] == 't') {
 					configtest = true;
 					ok = true;
 				}
 
-				if( ! ok )
-				{
+				if (argv[i][n] == 'V') {
+					Show_Version();
+					exit(1);
+				}
+
+				if (! ok) {
 					printf( "%s: invalid option \"-%c\"!\n", PACKAGE_NAME, argv[i][n] );
 					printf( "Try \"%s --help\" for more information.\n", PACKAGE_NAME );
 					exit( 1 );
@@ -397,7 +399,12 @@ Fill_Version( void )
 
 	strlcat( NGIRCd_VersionAddition, "IRCPLUS", sizeof NGIRCd_VersionAddition );
 #endif
+#ifdef WANT_IPV6
+	if (NGIRCd_VersionAddition[0])
+		strlcat(NGIRCd_VersionAddition, "+", sizeof(NGIRCd_VersionAddition));
 
+	strlcat(NGIRCd_VersionAddition, "IPv6", sizeof(NGIRCd_VersionAddition));
+#endif
 	if( NGIRCd_VersionAddition[0] )
 		strlcat( NGIRCd_VersionAddition, "-", sizeof( NGIRCd_VersionAddition ));
 
@@ -407,12 +414,9 @@ Fill_Version( void )
 	strlcat( NGIRCd_VersionAddition, "/", sizeof( NGIRCd_VersionAddition ));
 	strlcat( NGIRCd_VersionAddition, TARGET_OS, sizeof( NGIRCd_VersionAddition ));
 
-#ifdef CVSDATE
-	snprintf( NGIRCd_Version, sizeof NGIRCd_Version,"%s %s(%s)-%s", PACKAGE_NAME, PACKAGE_VERSION, CVSDATE, NGIRCd_VersionAddition);
-#else
-	snprintf( NGIRCd_Version, sizeof NGIRCd_Version, "%s %s-%s", PACKAGE_NAME, PACKAGE_VERSION, NGIRCd_VersionAddition);
-#endif
-} /* Fill_Version */
+	snprintf(NGIRCd_Version, sizeof NGIRCd_Version, "%s %s-%s",
+		 PACKAGE_NAME, PACKAGE_VERSION, NGIRCd_VersionAddition);
+	} /* Fill_Version */
 
 
 /**
@@ -422,24 +426,31 @@ GLOBAL void
 NGIRCd_Rehash( void )
 {
 	char old_name[CLIENT_ID_LEN];
+	unsigned old_nicklen;
 
 	Log( LOG_NOTICE|LOG_snotice, "Re-reading configuration NOW!" );
 	NGIRCd_SignalRehash = false;
 
+	/* Remember old server name and nick name length */
+	strlcpy( old_name, Conf_ServerName, sizeof old_name );
+	old_nicklen = Conf_MaxNickLength;
+
+	/* Re-read configuration ... */
+	if (!Conf_Rehash( ))
+		return;
+
 	/* Close down all listening sockets */
 	Conn_ExitListeners( );
 
-	/* Remember old server name */
-	strlcpy( old_name, Conf_ServerName, sizeof old_name );
-
-	/* Re-read configuration ... */
-	Conf_Rehash( );
-
-	/* Recover old server name: it can't be changed during run-time */
-	if( strcmp( old_name, Conf_ServerName ) != 0 )
-	{
-		strlcpy( Conf_ServerName, old_name, sizeof Conf_ServerName );
-		Log( LOG_ERR, "Can't change \"ServerName\" on runtime! Ignored new name." );
+	/* Recover old server name and nick name length: these values can't
+	 * be changed during run-time */
+	if (strcmp(old_name, Conf_ServerName) != 0 ) {
+		strlcpy(Conf_ServerName, old_name, sizeof Conf_ServerName);
+		Log(LOG_ERR, "Can't change \"ServerName\" on runtime! Ignored new name.");
+	}
+	if (old_nicklen != Conf_MaxNickLength) {
+		Conf_MaxNickLength = old_nicklen;
+		Log(LOG_ERR, "Can't change \"MaxNickLength\" on runtime! Ignored new value.");
 	}
 
 	/* Create new pre-defined channels */
@@ -546,7 +557,7 @@ static void
 Show_Version( void )
 {
 	puts( NGIRCd_Version );
-	puts( "Copyright (c)2001-2007 Alexander Barton (<alex@barton.de>) and Contributors." );
+	puts( "Copyright (c)2001-2008 Alexander Barton (<alex@barton.de>) and Contributors." );
 	puts( "Homepage: <http://ngircd.barton.de/>\n" );
 	puts( "This is free software; see the source for copying conditions. There is NO" );
 	puts( "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." );
@@ -571,8 +582,8 @@ Show_Help( void )
 	puts( "  -s, --sniffer      enable network sniffer and display all IRC traffic" );
 #endif
 	puts( "  -t, --configtest   read, validate and display configuration; then exit" );
- 	puts( "      --version      output version information and exit" );
-	puts( "      --help         display this help and exit" );
+	puts( "  -V, --version      output version information and exit" );
+	puts( "  -h, --help         display this help and exit" );
 } /* Show_Help */
 
 
@@ -797,7 +808,7 @@ NGIRCd_Init( bool NGIRCd_NoDaemon )
 			if( chdir( pwd->pw_dir ) == 0 ) 
 				Log( LOG_DEBUG, "Changed working directory to \"%s\" ...", pwd->pw_dir );
 			else 
-				Log( LOG_ERR, "Can't change working directory to \"%s\": %s",
+				Log( LOG_INFO, "Notice: Can't change working directory to \"%s\": %s",
 								pwd->pw_dir, strerror( errno ));
 		}
 	} else {
