@@ -1,6 +1,6 @@
 /*
  * ngIRCd -- The Next Generation IRC Daemon
- * Copyright (c)2001,2002 by Alexander Barton (alex@barton.de)
+ * Copyright (c)2001-2008 Alexander Barton (alex@barton.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,8 @@
  * Please read the file COPYING, README and AUTHORS for more information.
  */
 
-
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: parse.c,v 1.67 2006/04/23 10:37:27 fw Exp $";
-
 /**
  * @file
  * IRC command parser and validator.
@@ -48,11 +45,17 @@ static char UNUSED id[] = "$Id: parse.c,v 1.67 2006/04/23 10:37:27 fw Exp $";
 #include "irc-oper.h"
 #include "irc-server.h"
 #include "irc-write.h"
+#include "numeric.h"
 
 #include "exp.h"
 
+struct _NUMERIC {
+	int numeric;
+	bool (*function) PARAMS(( CLIENT *Client, REQUEST *Request ));
+};
 
-COMMAND My_Commands[] =
+
+static COMMAND My_Commands[] =
 {
 	{ "ADMIN", IRC_ADMIN, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
 	{ "AWAY", IRC_AWAY, CLIENT_USER, 0, 0, 0 },
@@ -61,6 +64,7 @@ COMMAND My_Commands[] =
 	{ "DISCONNECT", IRC_DISCONNECT, CLIENT_USER, 0, 0, 0 },
 	{ "ERROR", IRC_ERROR, 0xFFFF, 0, 0, 0 },
 	{ "HELP", IRC_HELP, CLIENT_USER, 0, 0, 0 },
+	{ "INFO", IRC_INFO, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
 	{ "INVITE", IRC_INVITE, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
 	{ "ISON", IRC_ISON, CLIENT_USER, 0, 0, 0 },
 	{ "JOIN", IRC_JOIN, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
@@ -87,12 +91,15 @@ COMMAND My_Commands[] =
 	{ "SERVER", IRC_SERVER, 0xFFFF, 0, 0, 0 },
 	{ "SQUIT", IRC_SQUIT, CLIENT_SERVER, 0, 0, 0 },
 	{ "STATS", IRC_STATS, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
+	{ "SUMMON", IRC_SUMMON, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
 	{ "TIME", IRC_TIME, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
 	{ "TOPIC", IRC_TOPIC, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
 	{ "TRACE", IRC_TRACE, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
 	{ "USER", IRC_USER, 0xFFFF, 0, 0, 0 },
 	{ "USERHOST", IRC_USERHOST, CLIENT_USER, 0, 0, 0 },
+	{ "USERS", IRC_USERS, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
 	{ "VERSION", IRC_VERSION, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
+	{ "WALLOPS", IRC_WALLOPS, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
 	{ "WHO", IRC_WHO, CLIENT_USER, 0, 0, 0 },
 	{ "WHOIS", IRC_WHOIS, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
 	{ "WHOWAS", IRC_WHOWAS, CLIENT_USER|CLIENT_SERVER, 0, 0, 0 },
@@ -102,7 +109,6 @@ COMMAND My_Commands[] =
 	{ NULL, NULL, 0x0, 0, 0, 0 } /* Ende-Marke */
 };
 
-
 static void Init_Request PARAMS(( REQUEST *Req ));
 
 static bool Validate_Prefix PARAMS(( CONN_ID Idx, REQUEST *Req, bool *Closed ));
@@ -111,6 +117,7 @@ static bool Validate_Args PARAMS(( CONN_ID Idx, REQUEST *Req, bool *Closed ));
 
 static bool Handle_Request PARAMS(( CONN_ID Idx, REQUEST *Req ));
 
+#define ARRAY_SIZE(x)	(sizeof(x)/sizeof((x)[0]))
 
 /**
  * Return the pointer to the global "IRC command structure".
@@ -328,27 +335,117 @@ Validate_Command( UNUSED CONN_ID Idx, UNUSED REQUEST *Req, bool *Closed )
 
 
 static bool
-Validate_Args( UNUSED CONN_ID Idx, UNUSED REQUEST *Req, bool *Closed )
+Validate_Args(CONN_ID Idx, REQUEST *Req, bool *Closed)
 {
+#ifdef STRICT_RFC
+	int i;
+#endif
+
 	assert( Idx >= 0 );
 	assert( Req != NULL );
 	*Closed = false;
 
+#ifdef STRICT_RFC
+	/* CR and LF are never allowed in command parameters.
+	 * But since we do accept lines terminated only with CR or LF in
+	 * "non-RFC-compliant mode" (besides the correct CR+LF combination),
+	 * this check can only trigger in "strict RFC" mode; therefore we
+	 * optimize it away otherwise ... */
+	for (i = 0; i < Req->argc; i++) {
+		if (strchr(Req->argv[i], '\r') || strchr(Req->argv[i], '\n')) {
+			Log(LOG_ERR,
+			    "Invalid character(s) in parameter (connection %d, command %s)!?",
+			    Idx, Req->command);
+			if (!Conn_WriteStr(Idx,
+					   "ERROR :Invalid character(s) in parameter!"))
+				*Closed = true;
+			return false;
+		}
+	}
+#endif
+
 	return true;
 } /* Validate_Args */
 
 
+/* Command is a status code ("numeric") from another server */
+static bool
+Handle_Numeric(CLIENT *client, REQUEST *Req)
+{
+	static const struct _NUMERIC Numerics[] = {
+		{ 005, IRC_Num_ISUPPORT },
+		{ 376, IRC_Num_ENDOFMOTD }
+	};
+	int i, num;
+	char str[LINE_LEN];
+	CLIENT *prefix, *target = NULL;
+
+	/* Determine target */
+	if (Req->argc > 0)
+		target = Client_Search(Req->argv[0]);
+
+	if (!target) {
+		/* Status code without target!? */
+		if (Req->argc > 0)
+			Log(LOG_WARNING,
+			    "Unknown target for status code %s: \"%s\"",
+			    Req->command, Req->argv[0]);
+		else
+			Log(LOG_WARNING,
+			    "Unknown target for status code %s!",
+			    Req->command);
+		return true;
+	}
+	if (target == Client_ThisServer()) {
+		/* This server is the target of the numeric */
+		num = atoi(Req->command);
+
+		for (i = 0; i < (int) ARRAY_SIZE(Numerics); i++) {
+			if (num == Numerics[i].numeric)
+				return Numerics[i].function(client, Req);
+		}
+
+		LogDebug("Ignored status code %s from \"%s\".",
+			 Req->command, Client_ID(client));
+		return true;
+	}
+
+	/* Determine source */
+	if (! Req->prefix[0]) {
+		/* Oops, no prefix!? */
+		Log(LOG_WARNING, "Got status code %s from \"%s\" without prefix!?",
+						Req->command, Client_ID(client));
+		return true;
+	}
+
+	prefix = Client_Search(Req->prefix);
+	if (! prefix) { /* Oops, unknown prefix!? */
+		Log(LOG_WARNING, "Got status code %s from unknown source: \"%s\"", Req->command, Req->prefix);
+		return true;
+	}
+
+	/* Forward status code */
+	strlcpy(str, Req->command, sizeof(str));
+	for (i = 0; i < Req->argc; i++) {
+		if (i < Req->argc - 1)
+			strlcat(str, " ", sizeof(str));
+		else
+			strlcat(str, " :", sizeof(str));
+		strlcat(str, Req->argv[i], sizeof(str));
+	}
+	return IRC_WriteStrClientPrefix(target, prefix, "%s", str);
+}
+
+
 static bool
 Handle_Request( CONN_ID Idx, REQUEST *Req )
 {
 	/* Client-Request verarbeiten. Bei einem schwerwiegenden Fehler
 	 * wird die Verbindung geschlossen und false geliefert. */
-
-	CLIENT *client, *target, *prefix;
-	char str[LINE_LEN];
-	bool result;
+	CLIENT *client;
+	bool result = true;
+	int client_type;
 	COMMAND *cmd;
-	int i;
 
 	assert( Idx >= 0 );
 	assert( Req != NULL );
@@ -357,90 +454,44 @@ Handle_Request( CONN_ID Idx, REQUEST *Req )
 	client = Conn_GetClient( Idx );
 	assert( client != NULL );
 
-	/* Statuscode? */
-	if(( Client_Type( client ) == CLIENT_SERVER ) && ( strlen( Req->command ) == 3 ) && ( atoi( Req->command ) > 100 ))
-	{
-		/* Command is a status code from an other server */
-
-		/* Determine target */
-		if( Req->argc > 0 ) target = Client_Search( Req->argv[0] );
-		else target = NULL;
-		if( ! target )
-		{
-			/* Status code without target!? */
-			if( Req->argc > 0 ) Log( LOG_WARNING, "Unknown target for status code %s: \"%s\"", Req->command, Req->argv[0] );
-			else Log( LOG_WARNING, "Unknown target for status code %s!", Req->command );
-			return true;
-		}
-		if( target == Client_ThisServer( ))
-		{
-			/* This server is the target, ignore it */
-			Log( LOG_DEBUG, "Ignored status code %s from \"%s\".", Req->command, Client_ID( client ));
-			return true;
-		}
-
-		/* Determine source */
-		if( ! Req->prefix[0] )
-		{
-			/* Oops, no prefix!? */
-			Log( LOG_WARNING, "Got status code %s from \"%s\" without prefix!?", Req->command, Client_ID( client ));
-			return true;
-		}
-		else prefix = Client_Search( Req->prefix );
-		if( ! prefix )
-		{
-			/* Oops, unknown prefix!? */
-			Log( LOG_WARNING, "Got status code %s from unknown source: \"%s\"", Req->command, Req->prefix );
-			return true;
-		}
-
-		/* Forward status code */
-		strlcpy( str, Req->command, sizeof( str ));
-		for( i = 0; i < Req->argc; i++ )
-		{
-			if( i < Req->argc - 1 ) strlcat( str, " ", sizeof( str ));
-			else strlcat( str, " :", sizeof( str ));
-			strlcat( str, Req->argv[i], sizeof( str ));
-		}
-		return IRC_WriteStrClientPrefix( target, prefix, "%s", str );
-	}
+	/* Numeric? */
+	client_type = Client_Type(client);
+	if ((client_type == CLIENT_SERVER ||
+	     client_type == CLIENT_UNKNOWNSERVER)
+	    && strlen(Req->command) == 3 && atoi(Req->command) > 1)
+		return Handle_Numeric(client, Req);
 
 	cmd = My_Commands;
-	while( cmd->name )
-	{
+	while (cmd->name) {
 		/* Befehl suchen */
-		if( strcasecmp( Req->command, cmd->name ) != 0 )
-		{
-			cmd++; continue;
+		if (strcasecmp(Req->command, cmd->name) != 0) {
+			cmd++;
+			continue;
 		}
 
-		if( Client_Type( client ) & cmd->type )
-		{
-			/* Command is allowed for this client: call it and count produced bytes */
-			Conn_ResetWCounter( );
-			result = (cmd->function)( client, Req );
-			cmd->bytes += Conn_WCounter( );
+		if (!(client_type & cmd->type))
+			return IRC_WriteStrClient(client, ERR_NOTREGISTERED_MSG, Client_ID(client));
 
-			/* Adjust counters */
-			if( Client_Type( client ) != CLIENT_SERVER ) cmd->lcount++;
-			else cmd->rcount++;
+		/* Command is allowed for this client: call it and count produced bytes */
+		Conn_ResetWCounter();
+		result = (cmd->function)(client, Req);
+		cmd->bytes += Conn_WCounter();
 
-			return result;
-		}
+		/* Adjust counters */
+		if (client_type != CLIENT_SERVER)
+			cmd->lcount++;
 		else
-		{
-			/* Befehl ist fuer diesen Client-Typ nicht erlaubt! */
-			return IRC_WriteStrClient( client, ERR_NOTREGISTERED_MSG, Client_ID( client ));
-		}
+			cmd->rcount++;
+		return result;
 	}
 
-	if( Client_Type( client ) != CLIENT_USER &&
-	    Client_Type( client ) != CLIENT_SERVER &&
-	    Client_Type( client ) != CLIENT_SERVICE )
+	if (client_type != CLIENT_USER &&
+	    client_type != CLIENT_SERVER &&
+	    client_type != CLIENT_SERVICE )
 		return true;
-	
+
 	/* Unknown command and registered connection: generate error: */
-	Log( LOG_DEBUG, "Connection %d: Unknown command \"%s\", %d %s,%s prefix.",
+	LogDebug("Connection %d: Unknown command \"%s\", %d %s,%s prefix.",
 			Client_Conn( client ), Req->command, Req->argc,
 			Req->argc == 1 ? "parameter" : "parameters",
 			Req->prefix ? "" : " no" );
@@ -449,10 +500,8 @@ Handle_Request( CONN_ID Idx, REQUEST *Req )
 		result = IRC_WriteStrClient(client, ERR_UNKNOWNCOMMAND_MSG,
 				Client_ID(client), Req->command);
 		Conn_SetPenalty(Idx, 1);
-		return result;
 	}
-
-	return true;
+	return result;
 } /* Handle_Request */
 
 
