@@ -8,7 +8,7 @@
  * (at your option) any later version.
  * Please read the file COPYING, README and AUTHORS for more information.
  *
- * $Id: resolve.h,v 1.13 2006/05/10 21:24:02 alex Exp $
+ * $Id: resolve.h,v 1.14 2008/02/26 22:04:17 fw Exp $
  *
  * Asynchronous resolver (header)
  */
@@ -18,6 +18,8 @@
 #define __resolve_h__
 
 #include "array.h"
+#include "tool.h"
+#include "ng_ipaddr.h"
 #include <netinet/in.h>
 
 /* This struct must not be accessed directly */
@@ -30,7 +32,7 @@ typedef struct _Res_Stat {
 #define Resolve_Getfd(x)		((x)->resolver_fd)
 #define Resolve_INPROGRESS(x)		((x)->resolver_fd >= 0)
 
-GLOBAL bool Resolve_Addr PARAMS(( RES_STAT *s, struct sockaddr_in *Addr, int identsock, void (*cbfunc)(int, short)));
+GLOBAL bool Resolve_Addr PARAMS(( RES_STAT *s, const ng_ipaddr_t *Addr, int identsock, void (*cbfunc)(int, short)));
 GLOBAL bool Resolve_Name PARAMS(( RES_STAT *s, const char *Host, void (*cbfunc)(int, short) ));
 GLOBAL size_t Resolve_Read PARAMS(( RES_STAT *s, void *buf, size_t buflen));
 GLOBAL void Resolve_Init PARAMS(( RES_STAT *s));
