@@ -8,7 +8,7 @@
  * (at your option) any later version.
  * Please read the file COPYING, README and AUTHORS for more information.
  *
- * $Id: tool.h,v 1.3 2005/03/19 18:43:53 fw Exp $
+ * $Id: tool.h,v 1.6 2008/02/26 22:04:18 fw Exp $
  *
  * Tool functions (Header)
  */
@@ -16,15 +16,19 @@
 
 #ifndef __tool_h__
 #define __tool_h__
+#include "portab.h"
 
+#ifdef HAVE_ARPA_INET_H
+# include <arpa/inet.h>
+#else
+# define PF_INET AF_INET
+#endif
 
 GLOBAL void ngt_TrimLastChr PARAMS((char *String, const char Chr ));
 
 GLOBAL void ngt_TrimStr PARAMS((char *String ));
 
 GLOBAL char *ngt_LowerStr PARAMS((char *String ));
-
-
 #endif
 
 
